Package: NeutroBasicDesignsAnalysis
Type: Package
Title: Neutrosophic Analysis of Completely Randomized Designs and
        Randomized Complete Block Designs
Version: 0.0.1
Authors@R: c(
    person(
      "Neethu", "R.S.",
      email = "neethumpras37@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
      "Boyina", "Devi Priyanka",
      email = "boyinadevipriyanka@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
      "Cini", "Varghese",
      email = "cini2204@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
      "Susheel Kumar", "Sarkar",
      email = "suskar982@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
      "Mohd", "Harun",
      email = "harun.agribhu@gmail.com",
      role = c("aut", "ctb")
    ),
    person(
  "Vinaykumar", "L.N.",
  email = "vinaymandya123@gmail.com",
  role = c("aut", "cre")
   ),
    person(
      "Anindita", "Datta",
      email = "aninditaiasri@gmail.com",
      role = c("aut", "ctb")
    )
  )
Maintainer: Vinaykumar L.N. <vinaymandya123@gmail.com>
Description: Provides neutrosophic statistical methods for interval-valued
    data from completely randomized and randomized complete block designs.
    Methods include neutrosophic analysis of variance, analysis of covariance,
    multivariate analysis of variance, pooled analysis of variance, Levene's
    test, and Aitken transformation. When the lower and upper bounds are equal
    (crisp data), the methods reduce to their corresponding classical
    statistical analyses. The basic concept of neutrosophic statistics is based
    on Smarandache (2014) <https://fs.unm.edu/NeutrosophicStatistics.pdf>,
    while the statistical analysis procedures implemented in this package are
    newly developed.
License: GPL (>= 3)
Encoding: UTF-8
Imports: MASS, stats
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-05 04:16:59 UTC; admin
Author: Neethu R.S. [aut, ctb],
  Boyina Devi Priyanka [aut, ctb],
  Cini Varghese [aut, ctb],
  Susheel Kumar Sarkar [aut, ctb],
  Mohd Harun [aut, ctb],
  Vinaykumar L.N. [aut, cre],
  Anindita Datta [aut, ctb]
Repository: CRAN
Date/Publication: 2026-08-09 08:40:13 UTC
Built: R 4.6.1; ; 2026-08-09 12:55:41 UTC; unix

# Generated by roxygen2: do not edit by hand

export(CRDnsANCOVA)
export(CRDnsANOVA)
export(CRDnsMANOVA)
export(PooledCRDnsANOVA)
export(PooledRBDnsANOVA)
export(RCBDnsANCOVA)
export(RCBDnsANOVA)
export(nsAitkenTransform)
export(nsLeveneTest)
importFrom(MASS,ginv)
importFrom(stats,model.matrix)
importFrom(stats,qf)
importFrom(stats,qt)

#' Neutrosophic Analysis of Variance for Completely Randomized Design
#'
#' Performs neutrosophic analysis of variance (NANOVA) for a
#' completely randomized design using interval-valued observations.
#'
#' @param Lower_y Matrix of lower bounds of the response variable.
#' @param Upper_y Matrix of upper bounds of the response variable.
#' @param design Matrix specifying treatment allocation.
#' @param alpha Significance level for LSD test.
#' @param verbose Logical. If TRUE, prints the analysis.
#'
#' @return A list containing the NANOVA table, treatment means,
#' pairwise comparisons, and LSD interval.
#'
#' @importFrom MASS ginv
#' @importFrom stats qf qt
#' @export
#' @examples
#' Lower_y <- matrix(c(
#'   9.5, 19.5, 29.5, 39.5,
#'   10.0, 20.0, 30.0, 40.0,
#'   10.5, 20.5, 30.5, 40.5,
#'   9.8, 19.8, 29.8, 39.8,
#'   10.2, 20.2, 30.2, 40.2
#' ), nrow = 5, byrow = TRUE)
#'
#' Upper_y <- matrix(c(
#'   10.5, 20.5, 30.5, 40.5,
#'   11.0, 21.0, 31.0, 41.0,
#'   11.5, 21.5, 31.5, 41.5,
#'   10.8, 20.8, 30.8, 40.8,
#'   11.2, 21.2, 31.2, 41.2
#' ), nrow = 5, byrow = TRUE)
#'
#' design <- matrix(c(
#'   1, 2, 3, 4,
#'   1, 2, 3, 4,
#'   1, 2, 3, 4,
#'   1, 2, 3, 4,
#'   1, 2, 3, 4
#' ), nrow = 5, byrow = TRUE)
#'
#' CRDnsANOVA(
#'   Lower_y = Lower_y,
#'   Upper_y = Upper_y,
#'   design = design,
#'   alpha = 0.05,
#'   verbose = TRUE
#' )


CRDnsANOVA <- function(Lower_y, Upper_y, design,
                       alpha = 0.05,
                       verbose = FALSE){

  #-------------------------------------------------------
  # Input Validation and Initialization
  #-------------------------------------------------------

  Lower_y <- as.matrix(Lower_y)
  Upper_y <- as.matrix(Upper_y)
  design  <- as.matrix(design)

  v <- max(design)
  b <- nrow(design)
  k <- ncol(design)

  rep_counts <- table(design)
  r <- unique(rep_counts)

  interval <- function(L, U){
    list(
      d = (L + U)/2,
      i = abs(U - L)/2
    )
  }

  n <- b * k

  #-------------------------------------------------------
  # Construction of Experimental Design and Model Matrices
  #-------------------------------------------------------

  trt_vec <- as.vector(t(design))

  Tmat <- matrix(0, n, v)

  for(i in 1:n){
    Tmat[i, trt_vec[i]] <- 1
  }

  mu <- matrix(1, n, 1)

  X <- cbind(Tmat, mu)

  #-------------------------------------------------------
  # Interval Decomposition of Responses
  #-------------------------------------------------------

  d_y <- (Lower_y + Upper_y)/2
  i_y <- abs(Upper_y - Lower_y)/2

  dy <- as.vector(t(d_y))
  iy <- as.vector(t(i_y))

  #-------------------------------------------------------
  # Computation of Overall Interval Means and
  # Centered Interval Responses
  #-------------------------------------------------------

  ybar_l <- mean(Lower_y)
  ybar_u <- mean(Upper_y)
  ybar_d <- mean(d_y)
  ybar_i <- mean(i_y)

  yminusbar_l <- d_y - ybar_d
  yminusbar_u <- yminusbar_l + i_y - ybar_i

  yminusbar_d <- (yminusbar_l + yminusbar_u)/2
  yminusbar_i <- abs(yminusbar_u - yminusbar_l)/2

  yminusbar_sq_l <- yminusbar_d * yminusbar_d

  yminusbar_sq_u <- yminusbar_sq_l +
    2 * yminusbar_d * yminusbar_i +
    yminusbar_i * yminusbar_i

  yminusbar_sq_d <- (yminusbar_sq_l + yminusbar_sq_u)/2
  yminusbar_sq_i <- abs(yminusbar_sq_u - yminusbar_sq_l)/2

  #-------------------------------------------------------
  # Estimation of Adjusted Treatment Effects
  #-------------------------------------------------------

  Z <- cbind(mu)

  M <- diag(nrow(X)) - Z %*% MASS::ginv(t(Z) %*% Z) %*% t(Z)

  Ct_adj <- t(Tmat) %*% M %*% Tmat

  bcdy_t_ad_d <- t(Tmat) %*% cbind(mu) %*%
    MASS::ginv(t(cbind(mu)) %*% cbind(mu)) %*%
    t(cbind(mu)) %*% dy

  bcdy_t_ad_i <- t(Tmat) %*% cbind(mu) %*%
    MASS::ginv(t(cbind(mu)) %*% cbind(mu)) %*%
    t(cbind(mu)) %*% iy

  Qy_ad_l <- t(Tmat) %*% dy - bcdy_t_ad_d

  Qy_ad_u <- Qy_ad_l[(1:v), ] +
    t(Tmat) %*% iy - bcdy_t_ad_i

  Qy_t_ad_d <- (Qy_ad_l + Qy_ad_u)/2
  Qy_t_ad_i <- abs(Qy_ad_l - Qy_ad_u)/2

  ay_t_ad_l <- MASS::ginv(Ct_adj) %*% Qy_t_ad_d

  ay_t_ad_u <- ay_t_ad_l + Qy_t_ad_i/b

  ay_t_ad_d <- (ay_t_ad_l + ay_t_ad_u)/2
  ay_t_ad_i <- abs(ay_t_ad_l - ay_t_ad_u)/2

  #-------------------------------------------------------
  # Computation of Interval Sums of Squares
  #-------------------------------------------------------

  toss_l <- sum(yminusbar_sq_l)
  toss_u <- sum(yminusbar_sq_u)
  toss_d <- sum(yminusbar_sq_d)
  toss_i <- abs(toss_l - toss_u)/2

  ss_t_ad_l <- sum(Qy_t_ad_d * ay_t_ad_d)

  ss_t_ad_u <- ss_t_ad_l +
    sum(Qy_t_ad_d * ay_t_ad_i +
          Qy_t_ad_i * ay_t_ad_d +
          Qy_t_ad_i * ay_t_ad_i)

  ss_t_ad_d <- (ss_t_ad_l + ss_t_ad_u)/2
  ss_t_ad_i <- abs(ss_t_ad_u - ss_t_ad_l)/2

  mdss_all_ad_l <- ss_t_ad_l
  mdss_all_ad_u <- ss_t_ad_u

  mdss_all_ad_d <- (mdss_all_ad_u + mdss_all_ad_l)/2
  mdss_all_ad_i <- abs(mdss_all_ad_u - mdss_all_ad_l)/2

  ess_all_ad_l <- toss_d - mdss_all_ad_d

  ess_all_ad_u <- ess_all_ad_l +
    toss_i - mdss_all_ad_i

  ess_all_ad_d <- (ess_all_ad_l + ess_all_ad_u)/2
  ess_all_ad_i <- abs(ess_all_ad_u - ess_all_ad_l)/2

  #-------------------------------------------------------
  # Computation of Interval Mean Squares and F-Statistics
  #-------------------------------------------------------

  SV <- c("Treatment","Error","Total")

  DF <- c(v - 1, n - v, n - 1)

  SSN <- list(
    c(ss_t_ad_l, ss_t_ad_u),
    c(ess_all_ad_l, ess_all_ad_u),
    c(toss_l, toss_u)
  )

  MSSN <- list(
    c(ss_t_ad_l/(v - 1), ss_t_ad_u/(v - 1)),
    c(ess_all_ad_l/(n - v), ess_all_ad_u/(n - v)),
    c(NA, NA)
  )

  F_t_ad_L <- (ss_t_ad_d/(v - 1)) / (ess_all_ad_d/(n - v))

  F_t_ad_U <- F_t_ad_L +
    (((ss_t_ad_d/(v - 1)) * (ess_all_ad_i/(n - v))) +
       ((ss_t_ad_i/(v - 1)) * (ess_all_ad_d/(n - v))) +
       ((ss_t_ad_i/(v - 1)) * (ess_all_ad_i/(n - v)))) /
    ((ess_all_ad_d/(n - v)) * (ess_all_ad_d/(n - v)))

  FN <- list(
    c(F_t_ad_L, F_t_ad_U),
    c(NA, NA),
    c(NA, NA)
  )

  format_interval <- function(x){
    if(any(is.na(x))) return("")
    paste0("[", round(x[1], 2), ", ", round(x[2], 2), "]")
  }

  SSN_fmt  <- sapply(SSN, format_interval)
  MSSN_fmt <- sapply(MSSN, format_interval)
  FN_fmt   <- sapply(FN, format_interval)

  nanova_table <- data.frame(
    SV = SV,
    DF = DF,
    SSN = SSN_fmt,
    MSSN = MSSN_fmt,
    FN = FN_fmt,
    stringsAsFactors = FALSE
  )
  #-------------------------------------------------------
  # Determination of Significance
  #-------------------------------------------------------

  Ftab_5   <- qf(0.95,  v - 1, n - v)
  Ftab_1   <- qf(0.99,  v - 1, n - v)
  Ftab_001 <- qf(0.999, v - 1, n - v)

  significance <- ifelse(
    F_t_ad_L > Ftab_001, "***",
    ifelse(
      F_t_ad_L > Ftab_1, "**",
      ifelse(
        F_t_ad_L > Ftab_5, "*", "NS"
      )
    )
  )

  nanova_table$Significance <- c(significance, "", "")


  #-------------------------------------------------------
  # Interval-Based LSD Analysis
  #-------------------------------------------------------

  dfe <- n - v

  MSE_d <- ess_all_ad_d / dfe
  MSE_i <- ess_all_ad_i / dfe

  SEd_d <- sqrt((2 * MSE_d) / r)
  SEd_i <- sqrt((2 * MSE_i) / r)

  tval <- qt(1 - alpha/2, dfe)

  LSD_l <- tval * SEd_d
  LSD_u <- LSD_l + tval * SEd_i

  #-------------------------------------------------------
  # Adjusted Treatment Means
  #-------------------------------------------------------

  trtmeans_d <- as.vector(ay_t_ad_d)
  trtmeans_i <- as.vector(ay_t_ad_i)

  grand_mean <- mean(dy)

  adjmeans_d <- grand_mean + trtmeans_d
  adjmeans_i <- trtmeans_i

  #-------------------------------------------------------
  # Pairwise Treatment Comparisons
  #-------------------------------------------------------

  comparison <- data.frame()

  for(i in 1:(v - 1)){

    for(j in (i + 1):v){

      Diff_l <- abs(adjmeans_d[i] - adjmeans_d[j])

      Diff_u <- Diff_l + adjmeans_i[i] + adjmeans_i[j]

      Lower <- min(Diff_l, Diff_u)
      Upper <- max(Diff_l, Diff_u)

      diff_interval <- paste0(
        "[",
        round(Lower, 2),
        ", ",
        round(Upper, 2),
        "]"
      )

      if(Upper < LSD_l){

        result <- "NS"

      }else if(Lower > LSD_u){

        result <- "S"

      }else{

        result <- "I"

      }

      rowdata <- data.frame(
        Treatment1 = i,
        Treatment2 = j,
        Difference = diff_interval,
        LSD = paste0(
          "[",
          round(LSD_l, 2),
          ", ",
          round(LSD_u, 2),
          "]"
        ),
        Result = result,
        stringsAsFactors = FALSE
      )

      comparison <- rbind(comparison, rowdata)

    }

  }

  #-------------------------------------------------------
  # Display Results
  #-------------------------------------------------------

  if(verbose){

    message("\nNeutrosophic Analysis of Variance Table")
    print(nanova_table, row.names = FALSE)

    if(significance != "NS"){

      message(
        "\nLSD Interval : [",
        round(LSD_l,4),
        ", ",
        round(LSD_u,4),
        "]"
      )

      message("\nTreatment Comparisons Using LSD")

      print(comparison, row.names = FALSE)

    }else{

      message("\nTreatment effect is non-significant.")
      message("Multiple comparison is not performed.")

    }

    message("\nSignificance Codes:")
    message("*** : Significant at p < 0.001")
    message("**  : Significant at p < 0.01")
    message("*   : Significant at p < 0.05")
    message("NS  : Non Significant")
    message("I   : Indeterminate")

  }

  #-------------------------------------------------------
  # Return Results
  #-------------------------------------------------------

  invisible(list(
    nanova_table = nanova_table,
    comparison = if(significance != "NS") comparison else NULL,
    LSD = if(significance != "NS")
      c(Lower = LSD_l, Upper = LSD_u)
    else NULL
  ))

}

#' Pooled Neutrosophic ANOVA for a Completely Randomized Design
#'
#' Performs pooled neutrosophic analysis of variance for a completely
#' randomized design.
#'
#' @param Lower_y Numeric matrix of lower response bounds. Columns must be
#'   arranged in consecutive year or location blocks, each containing all treatments.
#' @param Upper_y Numeric matrix of upper response bounds.
#' @param design Numeric treatment-layout matrix with the same dimensions as
#'   `Lower_y`.
#' @param alpha Significance level. Default is `0.05`.
#' @param verbose Logical; print results when `TRUE`.
#'
#' @return A list containing the initial and final NANOVA tables, interaction
#'   test, decision, and pooled MSE when applicable.
#'
#' @importFrom MASS ginv
#' @importFrom stats qf
#'
#' @examples
#' # Three replications, three treatments, and two years.
#' # Columns 1-3 are Year 1; columns 4-6 are Year 2.
#'
#' Lower_y <- matrix(c(
#'   20, 25, 30, 21, 26, 31,
#'   22, 27, 32, 23, 28, 33,
#'   21, 26, 31, 22, 27, 32
#' ), nrow = 3, byrow = TRUE)
#'
#' Upper_y <- matrix(c(
#'   22, 27, 32, 23, 28, 33,
#'   24, 29, 34, 25, 30, 35,
#'   23, 28, 33, 24, 29, 34
#' ), nrow = 3, byrow = TRUE)
#'
#' # Treatment layout:
#' # Year 1 = Treatments 1, 2, 3
#' # Year 2 = Treatments 1, 2, 3
#' design <- matrix(c(
#'   1, 2, 3, 1, 2, 3,
#'   1, 2, 3, 1, 2, 3,
#'   1, 2, 3, 1, 2, 3
#' ), nrow = 3, byrow = TRUE)
#'
#' result <- PooledCRDnsANOVA(
#'   Lower_y = Lower_y,
#'   Upper_y = Upper_y,
#'   design = design,
#'   alpha = 0.05,
#'   verbose = TRUE
#' )
#'
#' result$NANOVA.table
#' @export

PooledCRDnsANOVA <- function(Lower_y, Upper_y, design,
                             alpha = 0.05, verbose = FALSE) {

  Lower_y <- as.matrix(Lower_y)
  Upper_y <- as.matrix(Upper_y)
  design <- as.matrix(design)

  stopifnot(
    is.numeric(Lower_y),
    is.numeric(Upper_y),
    is.numeric(design),
    identical(dim(Lower_y), dim(Upper_y)),
    identical(dim(Lower_y), dim(design)),
    alpha > 0,
    alpha < 1
  )

  if (any(Lower_y > Upper_y)) {
    stop("Each value in Lower_y must be less than or equal to Upper_y.")
  }

  b <- nrow(design)                      # replications per treatment/year
  k <- ncol(design)
  v <- length(unique(as.vector(design))) # treatments

  treatment_counts <- table(design[1, ])

  if (length(treatment_counts) != v ||
      length(unique(as.numeric(treatment_counts))) != 1) {
    stop("Each treatment must occur equally often in each row of design.")
  }

  r <- as.integer(treatment_counts[1])   # years

  if (k != r * v) {
    stop("Columns must be ordered as consecutive year blocks.")
  }

  n <- b * k

  d_y <- (Lower_y + Upper_y) / 2
  i_y <- abs(Upper_y - Lower_y) / 2

  # Vectorisation: Year 1 observations, then Year 2, and so on.
  year_cols <- split(seq_len(k), ceiling(seq_len(k) / v))

  dy <- unlist(lapply(
    year_cols,
    function(cols) c(t(d_y[, cols, drop = FALSE]))
  ))

  iy <- unlist(lapply(
    year_cols,
    function(cols) c(t(i_y[, cols, drop = FALSE]))
  ))

  trt_labels <- unlist(lapply(
    year_cols,
    function(cols) c(t(design[, cols, drop = FALSE]))
  ))

  trt_levels <- sort(unique(as.vector(design)))
  trt_id <- match(trt_labels, trt_levels)

  Tmat <- matrix(0, nrow = n, ncol = v)
  Tmat[cbind(seq_len(n), trt_id)] <- 1

  Rmat <- matrix(0, nrow = n, ncol = r)
  year_id <- rep(seq_len(r), each = b * v)
  Rmat[cbind(seq_len(n), year_id)] <- 1

  mu <- matrix(1, nrow = n, ncol = 1)

  TYmat <- t(vapply(
    seq_len(n),
    function(i) kronecker(Rmat[i, ], Tmat[i, ]),
    numeric(r * v)
  ))

  # Total sum of squares
  ybar_d <- mean(d_y)
  ybar_i <- mean(i_y)

  yminusbar_l <- d_y - ybar_d
  yminusbar_u <- yminusbar_l + i_y - ybar_i

  yminusbar_d <- (yminusbar_l + yminusbar_u) / 2
  yminusbar_i <- abs(yminusbar_u - yminusbar_l) / 2

  total_sq_l <- yminusbar_d^2

  total_sq_u <- total_sq_l +
    2 * yminusbar_d * yminusbar_i +
    yminusbar_i^2

  toss_l <- sum(total_sq_l)
  toss_u <- sum(total_sq_u)
  toss_d <- (toss_l + toss_u) / 2
  toss_i <- abs(toss_u - toss_l) / 2

  interval_ss <- function(A, Z) {

    M <- diag(n) -
      Z %*%
      MASS::ginv(t(Z) %*% Z) %*%
      t(Z)

    C <- t(A) %*% M %*% A

    Q_l <- t(A) %*% dy -
      t(A) %*% Z %*%
      MASS::ginv(t(Z) %*% Z) %*%
      t(Z) %*% dy

    Q_u <- Q_l +
      t(A) %*% iy -
      t(A) %*% Z %*%
      MASS::ginv(t(Z) %*% Z) %*%
      t(Z) %*% iy

    Q_d <- (Q_l + Q_u) / 2
    Q_i <- abs(Q_u - Q_l) / 2

    a_l <- MASS::ginv(C) %*% Q_d
    a_u <- a_l + MASS::ginv(C) %*% Q_i

    a_d <- (a_l + a_u) / 2
    a_i <- abs(a_u - a_l) / 2

    ss_l <- sum(Q_d * a_d)

    ss_u <- ss_l +
      sum(
        Q_d * a_i +
          Q_i * a_d +
          Q_i * a_i
      )

    list(
      lower = ss_l,
      upper = ss_u,
      d = (ss_l + ss_u) / 2,
      i = abs(ss_u - ss_l) / 2
    )
  }

  # Main-effect sums of squares adjusted for the other main effect
  ss_treatment <- interval_ss(Tmat, cbind(mu, Rmat))
  ss_year <- interval_ss(Rmat, cbind(mu, Tmat))

  # Treatment Year interaction sum of squares
  ss_interaction <- interval_ss(TYmat, cbind(mu, Tmat, Rmat))

  # Error sum of squares
  model_ss_l <- ss_treatment$d + ss_year$d + ss_interaction$d

  model_ss_u <- model_ss_l +
    ss_treatment$i +
    ss_year$i +
    ss_interaction$i

  model_ss_d <- (model_ss_l + model_ss_u) / 2
  model_ss_i <- abs(model_ss_u - model_ss_l) / 2

  error_l <- toss_d - model_ss_d
  error_u <- error_l + toss_i - model_ss_i

  error_d <- (error_l + error_u) / 2
  error_i <- abs(error_u - error_l) / 2

  # Degrees of freedom for pooled CRD
  df_year <- r - 1
  df_treatment <- v - 1
  df_interaction <- (r - 1) * (v - 1)
  df_error <- r * v * (b - 1)
  df_total <- n - 1

  ms_interval <- function(ss, df) {
    list(
      lower = ss$lower / df,
      upper = ss$upper / df,
      d = ss$d / df,
      i = ss$i / df
    )
  }

  ms_year <- ms_interval(ss_year, df_year)
  ms_treatment <- ms_interval(ss_treatment, df_treatment)
  ms_interaction <- ms_interval(ss_interaction, df_interaction)

  ms_error <- list(
    lower = error_l / df_error,
    upper = error_u / df_error,
    d = error_d / df_error,
    i = error_i / df_error
  )

  f_interval <- function(numerator_ms, denominator_ms) {

    F_l <- numerator_ms$d / denominator_ms$d

    F_u <- F_l +
      (
        numerator_ms$d * denominator_ms$i +
          numerator_ms$i * denominator_ms$d +
          numerator_ms$i * denominator_ms$i
      ) /
      (denominator_ms$d^2)

    c(Lower = F_l, Upper = F_u)
  }

  # First test: Treatment Year interaction against Error
  F_interaction <- f_interval(ms_interaction, ms_error)

  F_critical_interaction <- stats::qf(
    1 - alpha,
    df1 = df_interaction,
    df2 = df_error
  )

  interaction_decision <- if (F_interaction["Lower"] > F_critical_interaction) {
    "Significant"
  } else if (F_interaction["Upper"] <= F_critical_interaction) {
    "Not Significant"
  } else {
    "Indeterminate"
  }

  # Significant interaction: use MS(Treatment Year).
  # Non-significant interaction: pool Interaction and Error.
  if (interaction_decision == "Not Significant") {

    df_pooled_error <- df_interaction + df_error

    pooled_error_l <- ss_interaction$lower + error_l
    pooled_error_u <- ss_interaction$upper + error_u
    pooled_error_d <- (pooled_error_l + pooled_error_u) / 2
    pooled_error_i <- abs(pooled_error_u - pooled_error_l) / 2

    denominator_ms <- list(
      lower = pooled_error_l / df_pooled_error,
      upper = pooled_error_u / df_pooled_error,
      d = pooled_error_d / df_pooled_error,
      i = pooled_error_i / df_pooled_error
    )

    denominator_name <- "Pooled Error (Treatment*Year + Error)"

    F_year <- f_interval(ms_year, denominator_ms)
    F_treatment <- f_interval(ms_treatment, denominator_ms)

    final_table <- data.frame(
      Source = c(
        "Year",
        "Treatment",
        "Pooled Error (Treatment*Year + Error)",
        "Total"
      ),
      DF = c(
        df_year,
        df_treatment,
        df_pooled_error,
        df_total
      ),
      SSN = c(
        sprintf("[%.2f, %.2f]", ss_year$lower, ss_year$upper),
        sprintf("[%.2f, %.2f]", ss_treatment$lower, ss_treatment$upper),
        sprintf("[%.2f, %.2f]", pooled_error_l, pooled_error_u),
        sprintf("[%.2f, %.2f]", toss_l, toss_u)
      ),
      MSSN = c(
        sprintf("[%.2f, %.2f]", ms_year$lower, ms_year$upper),
        sprintf("[%.2f, %.2f]", ms_treatment$lower, ms_treatment$upper),
        sprintf("[%.2f, %.2f]", denominator_ms$lower, denominator_ms$upper),
        ""
      ),
      FN = c(
        sprintf("[%.2f, %.2f]", F_year[1], F_year[2]),
        sprintf("[%.2f, %.2f]", F_treatment[1], F_treatment[2]),
        "",
        ""
      ),
      stringsAsFactors = FALSE
    )

  } else {

    denominator_ms <- ms_interaction
    denominator_name <- "Treatment*Year"

    F_year <- f_interval(ms_year, denominator_ms)
    F_treatment <- f_interval(ms_treatment, denominator_ms)

    final_table <- data.frame(
      Source = c(
        "Year",
        "Treatment",
        "Treatment*Year",
        "Error",
        "Total"
      ),
      DF = c(
        df_year,
        df_treatment,
        df_interaction,
        df_error,
        df_total
      ),
      SSN = c(
        sprintf("[%.2f, %.2f]", ss_year$lower, ss_year$upper),
        sprintf("[%.2f, %.2f]", ss_treatment$lower, ss_treatment$upper),
        sprintf("[%.2f, %.2f]", ss_interaction$lower, ss_interaction$upper),
        sprintf("[%.2f, %.2f]", error_l, error_u),
        sprintf("[%.2f, %.2f]", toss_l, toss_u)
      ),
      MSSN = c(
        sprintf("[%.2f, %.2f]", ms_year$lower, ms_year$upper),
        sprintf("[%.2f, %.2f]", ms_treatment$lower, ms_treatment$upper),
        sprintf("[%.2f, %.2f]", ms_interaction$lower, ms_interaction$upper),
        sprintf("[%.2f, %.2f]", ms_error$lower, ms_error$upper),
        ""
      ),
      FN = c(
        sprintf("[%.2f, %.2f]", F_year[1], F_year[2]),
        sprintf("[%.2f, %.2f]", F_treatment[1], F_treatment[2]),
        sprintf("[%.2f, %.2f]", F_interaction[1], F_interaction[2]),
        "",
        ""
      ),
      stringsAsFactors = FALSE
    )
  }

  interaction_table <- data.frame(
    Source = "Treatment*Year",
    DF = df_interaction,
    F_Neutrosophic = sprintf(
      "[%.2f, %.2f]",
      F_interaction[1],
      F_interaction[2]
    ),
    F_Critical = round(F_critical_interaction, 4),
    Decision = interaction_decision,
    check.names = FALSE
  )

  if (verbose) {
    cat("\nTreatment Year Interaction Test\n")
    print(interaction_table, row.names = FALSE)

    cat("\nFinal Pooled Neutrosophic CRD ANOVA\n")
    print(final_table, row.names = FALSE)

    cat("\nDenominator used for Year and Treatment:", denominator_name, "\n")
  }

  invisible(list(
    interaction.table = interaction_table,
    NANOVA.table = final_table,
    Interaction.F = F_interaction,
    Interaction.F.critical = F_critical_interaction,
    Interaction.decision = interaction_decision,
    denominator = denominator_name,
    denominator.MSE = c(
      Lower = denominator_ms$lower,
      Upper = denominator_ms$upper
    )
  ))
}

#' Neutrosophic Levene's Test for Interval-Valued Data
#'
#' Tests homogeneity of variances for interval-valued observations using
#' neutrosophic lower and upper test-statistic bounds.
#'
#' @param Lower_y Numeric matrix of lower interval bounds.
#' @param Upper_y Numeric matrix of upper interval bounds; same dimensions as
#'   `Lower_y`.
#' @param design Treatment-design matrix. Treatment labels in each row must be
#'   arranged in consecutive year blocks.
#' @param alpha Significance level. Default is `0.05`.
#' @param verbose Logical; print the test table when `TRUE`.
#'
#' @return An invisible list containing the Levene statistic interval,
#'   critical F value, homogeneity decision, and ANOVA table.
#'
#'@importFrom MASS ginv
#' @importFrom stats qf qt model.matrix
#'
#'@examples
#' # Two replications, three treatments, and two years.
#' # Columns are ordered as:
#' # Year 1 (T1, T2, T3), Year 2 (T1, T2, T3).
#'
#' Lower_y <- matrix(c(
#'   10, 12, 11, 13, 14, 12,
#'    9, 11, 10, 12, 13, 11
#' ), nrow = 2, byrow = TRUE)
#'
#' Upper_y <- matrix(c(
#'   12, 14, 13, 15, 16, 14,
#'   11, 13, 12, 14, 15, 13
#' ), nrow = 2, byrow = TRUE)
#'
#' design <- matrix(c(
#'   1, 2, 3, 1, 2, 3,
#'   1, 2, 3, 1, 2, 3
#' ), nrow = 2, byrow = TRUE)
#'
#' levene_result <- nsLeveneTest(
#'   Lower_y = Lower_y,
#'   Upper_y = Upper_y,
#'   design = design,
#'   alpha = 0.05,
#'   verbose = TRUE
#' )
#'
#' levene_result$Homogeneity
#'
#' @importFrom MASS ginv
#' @export


nsLeveneTest <- function(Lower_y, Upper_y, design,
                        alpha = 0.05, verbose = TRUE) {

  stopifnot(
    is.matrix(Lower_y),
    is.matrix(Upper_y),
    is.matrix(design),
    identical(dim(Lower_y), dim(Upper_y)),
    identical(dim(Lower_y), dim(design)),
    alpha > 0, alpha < 1
  )

  # Design dimensions
  b <- nrow(design)                  # replications
  k <- ncol(design)                  # total columns
  v <- length(unique(as.vector(design)))  # treatments

  # Number of years = repetitions of each treatment in one design row
  treatment.counts <- table(design[1, ])

  if (length(unique(as.numeric(treatment.counts))) != 1) {
    stop("Each treatment must occur equally often in each row of 'design'.")
  }

  r <- as.integer(treatment.counts[1])  # years

  if (k != r * v) {
    stop("Columns must contain r consecutive year blocks, each with v treatments.")
  }

  N <- b * k

  if (N <= r) {
    stop("Insufficient observations: N must be greater than the number of years.")
  }

  # Determinate and indeterminate components
  datad <- (Lower_y + Upper_y) / 2
  datai <- abs(Upper_y - Lower_y) / 2

  # Year means: assumes columns are ordered year-by-year
  year.mean_d <- vapply(
    seq_len(r),
    function(j) {
      cols <- ((j - 1) * v + 1):(j * v)
      mean(datad[, cols, drop = FALSE])
    },
    numeric(1)
  )

  year.mean_i <- vapply(
    seq_len(r),
    function(j) {
      cols <- ((j - 1) * v + 1):(j * v)
      mean(datai[, cols, drop = FALSE])
    },
    numeric(1)
  )

  # Matrices of year means, aligned with Lower_y and Upper_y
  mean.mat_d <- matrix(
    rep(year.mean_d, each = b * v),
    nrow = b, ncol = k, byrow = FALSE
  )

  mean.mat_i <- matrix(
    rep(year.mean_i, each = b * v),
    nrow = b, ncol = k, byrow = FALSE
  )

  # Absolute deviations: retains your specified neutrosophic formulation
  zl <- abs(datad - mean.mat_d)
  zu <- zl + datai - mean.mat_i

  d_y <- (zl + zu) / 2
  i_y <- abs(zu - zl) / 2

  # Vectorization is column-wise, matching the year-block arrangement
  d <- as.vector(d_y)
  i <- as.vector(i_y)

  # Year membership design matrix: replaces undefined Xyear
  year <- factor(rep(seq_len(r), each = b * v))
  Xyear <- model.matrix(~ year - 1)

  PX <- Xyear %*% MASS::ginv(t(Xyear) %*% Xyear) %*% t(Xyear)

  J <- matrix(1, nrow = N, ncol = N)
  P1 <- J / N
  I <- diag(N)

  #-------------------------------------------------------
  # Between-year sum of squares
  #-------------------------------------------------------
  px1 <- PX - P1

  zdashpx1L <- t(d) %*% px1
  zdashpx1U <- zdashpx1L + t(i) %*% px1

  zdashpx1d <- (zdashpx1L + zdashpx1U) / 2
  zdashpx1i <- abs(zdashpx1U - zdashpx1L) / 2

  ssbL <- sum(zdashpx1d * d)
  ssbU <- ssbL +
    sum(zdashpx1i * d) +
    sum(zdashpx1d * i) +
    sum(zdashpx1i * i)

  ssbd <- (ssbL + ssbU) / 2
  ssbi <- abs(ssbU - ssbL) / 2

  mssbd <- ssbd / (r - 1)
  mssbi <- ssbi / (r - 1)

  #-------------------------------------------------------
  # Within-year sum of squares
  #-------------------------------------------------------
  p1x <- I - PX

  zdashp1xL <- t(d) %*% p1x
  zdashp1xU <- zdashp1xL + t(i) %*% p1x

  zdashp1xd <- (zdashp1xL + zdashp1xU) / 2
  zdashp1xi <- abs(zdashp1xU - zdashp1xL) / 2

  sswL <- sum(zdashp1xd * d)
  sswU <- sswL +
    sum(zdashp1xi * d) +
    sum(zdashp1xd * i) +
    sum(zdashp1xi * i)

  sswd <- (sswL + sswU) / 2
  sswi <- abs(sswU - sswL) / 2

  msswd <- sswd / (N - r)
  msswi <- sswi / (N - r)

  #-------------------------------------------------------
  # Neutrosophic Levene's statistic
  #-------------------------------------------------------
  WL <- mssbd / msswd

  WU <- WL +
    (mssbd * msswi + mssbi * msswd + mssbi * msswi) /
    (msswd * msswd)

  #-------------------------------------------------------
  # Homogeneity decision
  #-------------------------------------------------------
  F_critical <- stats::qf(
    1 - alpha,
    df1 = r - 1,
    df2 = N - r
  )

  Homogeneity <- if (WL > F_critical) {
    "Not Homogeneous"
  } else if (WU <= F_critical) {
    "Homogeneous"
  } else {
    "Indeterminate"
  }

  Levene.table <- data.frame(
    Source = c("Between Years", "Within Years"),
    DF = c(r - 1, N - r),
    `Neutrosophic Levene Statistic` = c(
      sprintf("[%.4f, %.4f]", WL, WU),
      "-"
    ),
    `F Critical` = c(sprintf("%.4f", F_critical), "-"),
    Decision = c(Homogeneity, "-"),
    check.names = FALSE
  )

  if (verbose) {
    cat("\n")
    cat("============================================================\n")
    cat("        NEUTROSOPHIC LEVENE'S TEST OF HOMOGENEITY\n")
    cat("============================================================\n\n")
    print(Levene.table, row.names = FALSE, right = FALSE)
    cat("\n")
  }

  invisible(list(
    Levene.table = Levene.table,
    Levene.Statistic = c(Lower = WL, Upper = WU),
    F.critical = F_critical,
    Homogeneity = Homogeneity,
    Years = r,
    Treatments = v,
    Replications = b
  ))
}

#' Neutrosophic MANOVA for a Completely Randomized Design
#'
#' Performs neutrosophic multivariate analysis of variance (MANOVA) for
#' interval-valued multivariate observations under a completely randomized
#' design using Wilks' Lambda as the multivariate test criterion.
#'
#' @param Lower_y A numeric matrix containing the lower bounds of the response
#'   variables. Rows represent observations and columns represent response
#'   variables.
#' @param Upper_y A numeric matrix containing the upper bounds of the response
#'   variables. Must have the same dimensions as `Lower_y`.
#' @param group A vector or factor identifying the treatment group for each
#'   observation. Its length must equal `nrow(Lower_y)`.
#' @param alpha Significance level used to obtain the critical F value.
#'   Defaults to `0.05`.
#' @param verbose Logical; if `TRUE`, prints the neutrosophic MANOVA table.
#'   Defaults to `TRUE`.
#'
#' @return
#' A list containing the results of the neutrosophic MANOVA with the following
#' components:
#' \describe{
#'   \item{MANOVA.table}{A data frame containing the neutrosophic MANOVA table,
#'   including the source of variation, degrees of freedom, Wilks' Lambda
#'   interval, approximate F-statistic interval, and decision.}
#'   \item{Wilks.Lambda}{A named numeric vector containing the lower and upper
#'   bounds of Wilks' Lambda.}
#'   \item{Approx.F}{A named numeric vector containing the lower and upper
#'   bounds of the approximate F-statistic.}
#'   \item{F.critical}{The critical F value at the specified significance
#'   level.}
#'   \item{Decision}{The hypothesis testing decision ("Significant",
#'   "Not Significant", or "Indeterminate").}
#'   \item{treatment_ss}{A list containing the lower, upper, determinate, and
#'   indeterminate treatment sum of squares matrices.}
#'   \item{total_ss}{A list containing the lower, upper, determinate, and
#'   indeterminate total sum of squares matrices.}
#'   \item{residual_ss}{A list containing the lower, upper, determinate, and
#'   indeterminate residual sum of squares matrices.}
#' }
#'
#' @importFrom MASS ginv
#' @importFrom stats model.matrix qf
#'
#' @examples
#' YL <- matrix(c(8.27, 2.50, 5.30, 1.41,
#'                8.40, 6.27, 0.00, 3.42),
#'              ncol = 2, byrow = TRUE)
#'
#' YU <- matrix(c(9.73, 3.50, 6.70, 2.59,
#'                9.60, 7.73, 0.00, 4.58),
#'              ncol = 2, byrow = TRUE)
#'
#' g <- c(1, 1, 2, 2)
#'
#' fit <- CRDnsMANOVA(
#'   Lower_y = YL,
#'   Upper_y = YU,
#'   group = g
#' )
#'
#' fit$MANOVA.table
#' fit$Decision
#' @export

CRDnsMANOVA <- function(Lower_y, Upper_y, group,
                        alpha = 0.05, verbose = TRUE) {

  stopifnot(is.matrix(Lower_y), is.matrix(Upper_y),
            all(dim(Lower_y) == dim(Upper_y)),
            length(group) == nrow(Lower_y),
            alpha > 0, alpha < 1)

  N <- nrow(Lower_y)
  p <- ncol(Lower_y)
  group <- factor(group)
  g <- nlevels(group)
  n <- as.numeric(table(group))

  if (N <= g + 1) {
    stop("Wilks F approximation requires N > number of groups + 1.")
  }

  # Design matrices
  X1 <- model.matrix(~ group - 1)
  Ones <- rep(1, N)
  xt <- cbind(X1, Ones)

  C <- t(X1) %*% X1 -
    t(X1) %*% Ones %*%
    ginv(t(Ones) %*% Ones) %*%
    t(Ones) %*% X1
  C_inv <- ginv(C)

  # Determinate and indeterminate response matrices
  Yd <- (Lower_y + Upper_y) / 2
  Yi <- abs(Upper_y - Lower_y) / 2

  # Treatment Sum of Squares
  xdashyd <- t(xt) %*% Yd
  xdashyi <- t(xt) %*% Yi

  det_bcd <- t(X1) %*% Ones %*%
    ginv(t(Ones) %*% Ones) %*%
    xdashyd[nrow(xdashyd), , drop = FALSE]

  i_bcd <- t(X1) %*% Ones %*%
    ginv(t(Ones) %*% Ones) %*%
    xdashyi[nrow(xdashyi), , drop = FALSE]

  L_q <- xdashyd[seq_len(g), , drop = FALSE] - det_bcd
  U_q <- L_q + xdashyi[seq_len(g), , drop = FALSE] - i_bcd

  det_q <- (L_q + U_q) / 2
  i_q <- abs((U_q - L_q) / 2)

  L_tuo <- C_inv %*% det_q
  U_tuo <- L_tuo + C_inv %*% i_q

  det_tuo <- (L_tuo + U_tuo) / 2
  i_tuo <- abs((U_tuo - L_tuo) / 2)

  L_trtss <- t(det_q) %*% det_tuo
  U_trtss <- L_trtss +
    t(det_q) %*% i_tuo +
    t(i_q) %*% det_tuo +
    t(i_q) %*% i_tuo

  det_trtss <- (L_trtss + U_trtss) / 2
  i_trtss <- abs((U_trtss - L_trtss) / 2)

  # Total Sum of Squares
  L_ydashy <- t(Yd) %*% Yd
  U_ydashy <- L_ydashy + 2 * t(Yd) %*% Yi + t(Yi) %*% Yi

  det_ydashy <- (L_ydashy + U_ydashy) / 2
  i_ydashy <- abs((U_ydashy - L_ydashy) / 2)

  J <- matrix(1, N, N)

  L_ydashj <- t(Yd) %*% J
  U_ydashj <- t(Yi) %*% J + L_ydashj

  det_ydashj <- (L_ydashj + U_ydashj) / 2
  i_ydashj <- abs((U_ydashj - L_ydashj) / 2)

  L_ydashjy <- det_ydashj %*% Yd
  U_ydashjy <- L_ydashjy +
    det_ydashj %*% Yi +
    i_ydashj %*% Yd +
    i_ydashj %*% Yi

  det_ydashjy <- (L_ydashjy + U_ydashjy) / 2
  i_ydashjy <- abs((U_ydashjy - L_ydashjy) / 2)

  L_TSS <- det_ydashy - det_ydashjy / N
  U_TSS <- L_TSS + i_ydashy - i_ydashjy / N

  det_TSS <- (L_TSS + U_TSS) / 2
  i_TSS <- abs((U_TSS - L_TSS) / 2)

  # Residual Sum of Squares
  L_RSS <- det_TSS - det_trtss
  U_RSS <- L_RSS + i_TSS - i_trtss

  det_RSS <- (L_RSS + U_RSS) / 2
  i_RSS <- abs((U_RSS - L_RSS) / 2)

  # Wilks lambda interval
  L_w <- det(det_RSS) / det(det_TSS)

  U_w <- L_w +
    det(det_RSS %*% i_TSS +
          i_RSS %*% det_TSS +
          i_RSS %*% i_TSS) /
    det(det_TSS %*% det_TSS)

  # F-statistic interval
  L_F <- ((1 - sqrt(L_w)) / sqrt(L_w)) * ((N - g - 1) / (g - 1))
  U_F <- ((1 - sqrt(U_w)) / sqrt(U_w)) * ((N - g - 1) / (g - 1))

  #-------------------------------------------------------
  # Decision
  #-------------------------------------------------------
  F_critical <- stats::qf(
    1 - alpha,
    df1 = g - 1,
    df2 = N - g
  )

  if (L_F > F_critical) {
    Decision <- "Significant"
  } else if (U_F < F_critical) {
    Decision <- "Not Significant"
  } else {
    Decision <- "Indeterminate"
  }

  #-------------------------------------------------------
  # MANOVA Table
  #-------------------------------------------------------
  MANOVA.table <- data.frame(
    Source = c("Treatments", "Error", "Total"),
    DF = c(g - 1, N - g, N - 1),
    `Wilks' Lambda` = c(
      sprintf("[%.4f, %.4f]", L_w, U_w),
      "-", "-"
    ),
    `Approx. F` = c(
      sprintf("[%.4f, %.4f]", L_F, U_F),
      "-", "-"
    ),
    Decision = c(Decision, "-", "-"),
    check.names = FALSE
  )

  #-------------------------------------------------------
  # Printing
  #-------------------------------------------------------
  if (verbose) {
    cat("\n")
    cat("============================================================\n")
    cat("      NEUTROSOPHIC MULTIVARIATE ANALYSIS OF VARIANCE\n")
    cat("                 (Wilks' Lambda Criterion)\n")
    cat("============================================================\n\n")

    print(MANOVA.table, row.names = FALSE, right = FALSE)
    cat("\nF critical value:", round(F_critical, 4), "\n")
  }

  result = list(
    MANOVA.table = MANOVA.table,
    Wilks.Lambda = c(Lower = L_w, Upper = U_w),
    Approx.F = c(Lower = L_F, Upper = U_F),
    F.critical = F_critical,
    Decision = Decision,
    treatment_ss = list(
      lower = L_trtss, upper = U_trtss,
      determinate = det_trtss, indeterminate = i_trtss
    ),
    total_ss = list(
      lower = L_TSS, upper = U_TSS,
      determinate = det_TSS, indeterminate = i_TSS
    ),
    residual_ss = list(
      lower = L_RSS, upper = U_RSS,
      determinate = det_RSS, indeterminate = i_RSS
    )
  )

  if (verbose) {
    cat("Neutrosophic CRD MANOVA\n")
    cat("Groups:", g, "| Observations:", N, "| Responses:", p, "\n")
    cat("Wilks' Lambda interval:", L_w, "to", U_w, "\n")
    cat("F-statistic interval:", L_F, "to", U_F, "\n")
  }

  return(result)
}

#' Neutrosophic Analysis of Variance for Randomized Complete Block Design
#'
#' Performs Neutrosophic Analysis of Variance (NANOVA) for interval-valued
#' response data from a Randomized Complete Block Design (RCBD).
#'
#' @param Lower_y Numeric matrix of lower bounds of the response variable.
#' @param Upper_y Numeric matrix of upper bounds of the response variable.
#' @param design Numeric matrix representing the RCBD layout.
#' @param alpha Significance level for interval-based LSD test. Default is 0.05.
#' @param verbose Logical. If \code{TRUE}, prints the NANOVA table and LSD comparisons.
#'
#' @return A list containing the Neutrosophic ANOVA table, interval-based LSD
#' comparisons (if applicable), and the interval LSD.
#'
#' @examples
#' Lower_y <- matrix(c(
#'   120.230,125.488,132.987,127.086,127.672,128.013,
#'   122.594,121.009,123.969,120.358,120.424,122.197,
#'   121.183,130.671,128.794,114.863,122.595,122.073,
#'   127.620,124.532,132.893,125.528,125.850,127.550
#' ), nrow = 4, byrow = TRUE)
#'
#' Upper_y <- matrix(c(
#'   127.6967536,131.2116955,141.2127373,136.1540904,130.6884772,136.8474149,
#'   129.8264289,130.3314544,133.3113414,126.5063118,128.4362999,130.2714433,
#'   124.5068016,139.3287297,134.1060197,124.2774447,127.2248520,130.3948469,
#'   131.0638721,129.8884785,135.5666716,127.7580663,132.0178679,133.3903886
#' ), nrow = 4, byrow = TRUE)
#'
#' design <- matrix(c(
#'   1,2,3,4,5,6,
#'   1,2,3,4,5,6,
#'   1,2,3,4,5,6,
#'   1,2,3,4,5,6
#' ), nrow = 4, byrow = TRUE)
#'
#' RCBDnsANOVA(Lower_y, Upper_y, design)
#'
#' @importFrom MASS ginv
#' @importFrom stats qf qt
#' @export



RCBDnsANOVA <- function(Lower_y, Upper_y,
                        design,
                        alpha = 0.05,
                        verbose = FALSE){

  #-------------------------------------------------------
  # Input Validation and Initialization
  #-------------------------------------------------------

  Lower_y <- as.matrix(Lower_y)
  Upper_y <- as.matrix(Upper_y)
  design  <- as.matrix(design)

  v <- max(design)
  b <- nrow(design)
  k <- ncol(design)

  rep_counts <- table(design)
  r <- unique(rep_counts)

  interval <- function(L, U){
    list(
      d = (L + U)/2,
      i = abs(U - L)/2
    )
  }

  n <- b * k

  #-------------------------------------------------------
  # Construction of Experimental Design and Model Matrices
  #-------------------------------------------------------

  trt_vec <- as.vector(t(design))

  Tmat <- matrix(0, n, v)

  for(i in 1:n){
    Tmat[i, trt_vec[i]] <- 1
  }

  mu <- matrix(1, n, 1)

  rep_vec <- rep(1:r, each = b/r * k)

  Rmat <- matrix(0, n, r)

  for(i in 1:n){
    Rmat[i, rep_vec[i]] <- 1
  }

  X <- cbind(Tmat, mu, Rmat)

  #-------------------------------------------------------
  # Interval Decomposition of Responses
  #-------------------------------------------------------

  d_y <- (Lower_y + Upper_y)/2
  i_y <- abs(Upper_y - Lower_y)/2

  dy <- as.vector(t(d_y))
  iy <- as.vector(t(i_y))

  #-------------------------------------------------------
  # Computation of Overall Interval Means and
  # Centered Interval Responses
  #-------------------------------------------------------

  ybar_l <- mean(Lower_y)
  ybar_u <- mean(Upper_y)
  ybar_d <- mean(d_y)
  ybar_i <- mean(i_y)

  yminusbar_l <- d_y - ybar_d
  yminusbar_u <- yminusbar_l + i_y - ybar_i

  yminusbar_d <- (yminusbar_l + yminusbar_u)/2
  yminusbar_i <- abs(yminusbar_u - yminusbar_l)/2

  yminusbar_sq_l <- yminusbar_d * yminusbar_d

  yminusbar_sq_u <- yminusbar_sq_l +
    2 * yminusbar_d * yminusbar_i +
    yminusbar_i * yminusbar_i

  yminusbar_sq_d <- (yminusbar_sq_l + yminusbar_sq_u)/2
  yminusbar_sq_i <- abs(yminusbar_sq_u - yminusbar_sq_l)/2

  #-------------------------------------------------------
  # Computation of Total Interval Sum of Squares
  #-------------------------------------------------------

  toss_l <- sum(yminusbar_sq_l)
  toss_u <- sum(yminusbar_sq_u)
  toss_d <- sum(yminusbar_sq_d)
  toss_i <- abs(toss_l - toss_u)/2


  #-------------------------------------------------------
  # Estimation of Adjusted Treatment Effects
  #-------------------------------------------------------

  Z <- cbind(mu, Rmat)

  M <- diag(nrow(X)) - Z %*% MASS::ginv(t(Z) %*% Z) %*% t(Z)

  Ct_adj <- t(Tmat) %*% M %*% Tmat

  bcdy_t_ad_d <- t(Tmat) %*% cbind(mu, Rmat) %*%
    MASS::ginv(t(cbind(mu, Rmat)) %*% cbind(mu, Rmat)) %*%
    t(cbind(mu, Rmat)) %*% dy

  bcdy_t_ad_i <- t(Tmat) %*% cbind(mu, Rmat) %*%
    MASS::ginv(t(cbind(mu, Rmat)) %*% cbind(mu, Rmat)) %*%
    t(cbind(mu, Rmat)) %*% iy

  Qy_ad_l <- t(Tmat) %*% dy - bcdy_t_ad_d

  Qy_ad_u <- Qy_ad_l[(1:v), ] +
    t(Tmat) %*% iy - bcdy_t_ad_i

  Qy_t_ad_d <- (Qy_ad_l + Qy_ad_u)/2
  Qy_t_ad_i <- abs(Qy_ad_l - Qy_ad_u)/2

  ay_t_ad_l <- MASS::ginv(Ct_adj) %*% Qy_t_ad_d

  ay_t_ad_u <- ay_t_ad_l + Qy_t_ad_i/b

  ay_t_ad_d <- (ay_t_ad_l + ay_t_ad_u)/2
  ay_t_ad_i <- abs(ay_t_ad_l - ay_t_ad_u)/2

  #-------------------------------------------------------
  # Estimation of Adjusted Block Effects
  #-------------------------------------------------------

  Z <- cbind(mu, Tmat)

  M <- diag(nrow(X)) - Z %*% MASS::ginv(t(Z) %*% Z) %*% t(Z)

  Cb_adj <- t(Rmat) %*% M %*% Rmat

  bcdy_b_ad_d <- t(Rmat) %*% cbind(mu, Tmat) %*%
    MASS::ginv(t(cbind(mu, Tmat)) %*% cbind(mu, Tmat)) %*%
    t(cbind(mu, Tmat)) %*% dy

  bcdy_b_ad_i <- t(Rmat) %*% cbind(mu, Tmat) %*%
    MASS::ginv(t(cbind(mu, Tmat)) %*% cbind(mu, Tmat)) %*%
    t(cbind(mu, Tmat)) %*% iy

  Qy_ad_l <- t(Rmat) %*% dy - bcdy_b_ad_d

  Qy_ad_u <- Qy_ad_l[(1:b), ] +
    t(Rmat) %*% iy - bcdy_b_ad_i

  Qy_b_ad_d <- (Qy_ad_l + Qy_ad_u)/2
  Qy_b_ad_i <- abs(Qy_ad_l - Qy_ad_u)/2

  ay_b_ad_l <- MASS::ginv(Cb_adj) %*% Qy_b_ad_d

  ay_b_ad_u <- ay_b_ad_l + Qy_b_ad_i/v

  ay_b_ad_d <- (ay_b_ad_l + ay_b_ad_u)/2
  ay_b_ad_i <- abs(ay_b_ad_l - ay_b_ad_u)/2

  #-------------------------------------------------------
  # Computation of Interval Sums of Squares
  #-------------------------------------------------------

  ss_t_ad_l <- sum(Qy_t_ad_d * ay_t_ad_d)

  ss_t_ad_u <- ss_t_ad_l +
    sum(Qy_t_ad_d * ay_t_ad_i +
          Qy_t_ad_i * ay_t_ad_d +
          Qy_t_ad_i * ay_t_ad_i)

  ss_t_ad_d <- (ss_t_ad_l + ss_t_ad_u)/2
  ss_t_ad_i <- abs(ss_t_ad_u - ss_t_ad_l)/2

  ss_b_ad_l <- sum(Qy_b_ad_d * ay_b_ad_d)

  ss_b_ad_u <- ss_b_ad_l +
    sum(Qy_b_ad_d * ay_b_ad_i +
          Qy_b_ad_i * ay_b_ad_d +
          Qy_b_ad_i * ay_b_ad_i)

  ss_b_ad_d <- (ss_b_ad_l + ss_b_ad_u)/2
  ss_b_ad_i <- abs(ss_b_ad_u - ss_b_ad_l)/2

  mdss_all_ad_l <- ss_t_ad_d + ss_b_ad_d

  mdss_all_ad_u <- mdss_all_ad_l +
    ss_t_ad_i + ss_b_ad_i

  mdss_all_ad_d <- (mdss_all_ad_u + mdss_all_ad_l)/2
  mdss_all_ad_i <- abs(mdss_all_ad_u - mdss_all_ad_l)/2

  ess_all_ad_l <- toss_d - mdss_all_ad_d

  ess_all_ad_u <- ess_all_ad_l +
    toss_i - mdss_all_ad_i

  ess_all_ad_d <- (ess_all_ad_l + ess_all_ad_u)/2
  ess_all_ad_i <- abs(ess_all_ad_u - ess_all_ad_l)/2

  #-------------------------------------------------------
  # Computation of Interval Mean Squares and F-Statistics
  #-------------------------------------------------------

  SV <- c("Treatment", "Replication", "Error", "Total")

  DF <- c(v - 1, b - 1, (v - 1) * (b - 1), n - 1)

  SSN <- list(
    c(ss_t_ad_l, ss_t_ad_u),
    c(ss_b_ad_l, ss_b_ad_u),
    c(ess_all_ad_l, ess_all_ad_u),
    c(toss_l, toss_u)
  )

  MSSN <- list(
    c(ss_t_ad_l/(v - 1), ss_t_ad_u/(v - 1)),
    c(ss_b_ad_l/(b - 1), ss_b_ad_u/(b - 1)),
    c(ess_all_ad_l/((v - 1) * (b - 1)),
      ess_all_ad_u/((v - 1) * (b - 1))),
    c(NA, NA)
  )

  F_t_ad_L <- (ss_t_ad_d/(v - 1)) /
    (ess_all_ad_d/((v - 1) * (b - 1)))

  F_t_ad_U <- F_t_ad_L +
    (((ss_t_ad_d/(v - 1)) *
        (ess_all_ad_i/((v - 1) * (b - 1)))) +
       ((ss_t_ad_i/(v - 1)) *
          (ess_all_ad_d/((v - 1) * (b - 1)))) +
       ((ss_t_ad_i/(v - 1)) *
          (ess_all_ad_i/((v - 1) * (b - 1))))) /
    ((ess_all_ad_d/((v - 1) * (b - 1))) *
       (ess_all_ad_d/((v - 1) * (b - 1))))

  F_b_ad_L <- (ss_b_ad_d/(b - 1)) /
    (ess_all_ad_d/((v - 1) * (b - 1)))

  F_b_ad_U <- F_b_ad_L +
    (((ss_b_ad_d/(b - 1)) *
        (ess_all_ad_i/((v - 1) * (b - 1)))) +
       ((ss_b_ad_i/(b - 1)) *
          (ess_all_ad_d/((v - 1) * (b - 1)))) +
       ((ss_b_ad_i/(b - 1)) *
          (ess_all_ad_i/((v - 1) * (b - 1))))) /
    ((ess_all_ad_d/((v - 1) * (b - 1))) *
       (ess_all_ad_d/((v - 1) * (b - 1))))

  FN <- list(
    c(F_t_ad_L, F_t_ad_U),
    c(F_b_ad_L, F_b_ad_U),
    c(NA, NA),
    c(NA, NA)
  )

  format_interval <- function(x){
    if(any(is.na(x))) return("")
    paste0("[", round(x[1], 2), ", ", round(x[2], 2), "]")
  }

  SSN_fmt <- sapply(SSN, format_interval)
  MSSN_fmt <- sapply(MSSN, format_interval)
  FN_fmt <- sapply(FN, format_interval)

  nanova_table <- data.frame(
    SV = SV,
    DF = DF,
    SSN = SSN_fmt,
    MSSN = MSSN_fmt,
    FN = FN_fmt,
    stringsAsFactors = FALSE
  )
  #-------------------------------------------------------
  # Determination of Significance
  #-------------------------------------------------------

  Ftab5_t   <- qf(0.95, v-1, (v-1)*(b-1))
  Ftab1_t   <- qf(0.99, v-1, (v-1)*(b-1))
  Ftab001_t <- qf(0.999, v-1, (v-1)*(b-1))

  trt_significance <- ifelse(
    F_t_ad_L > Ftab001_t, "***",
    ifelse(
      F_t_ad_L > Ftab1_t, "**",
      ifelse(
        F_t_ad_L > Ftab5_t, "*", "NS"
      )
    )
  )

  Ftab5_b   <- qf(0.95, b-1, (v-1)*(b-1))
  Ftab1_b   <- qf(0.99, b-1, (v-1)*(b-1))
  Ftab001_b <- qf(0.999, b-1, (v-1)*(b-1))

  rep_significance <- ifelse(
    F_b_ad_L > Ftab001_b, "***",
    ifelse(
      F_b_ad_L > Ftab1_b, "**",
      ifelse(
        F_b_ad_L > Ftab5_b, "*", "NS"
      )
    )
  )

  nanova_table$Significance <- c(
    trt_significance,
    rep_significance,
    "",
    ""
  )

  #-------------------------------------------------------
  # Interval-Based LSD Analysis
  #-------------------------------------------------------

  if(trt_significance != "NS"){

    dfe <- (v-1)*(b-1)

    MSE_d <- ess_all_ad_d/dfe
    MSE_i <- ess_all_ad_i/dfe

    SEd_d <- sqrt((2*MSE_d)/r)
    SEd_i <- sqrt((2*MSE_i)/r)

    tval <- qt(1-alpha/2,dfe)

    LSD_l <- tval*SEd_d
    LSD_u <- LSD_l+tval*SEd_i

    grand_mean <- mean(dy)

    adjmeans_d <- grand_mean + as.vector(ay_t_ad_d)
    adjmeans_i <- as.vector(ay_t_ad_i)

    comparison <- data.frame()

    for(i in 1:(v-1)){

      for(j in (i+1):v){

        Diff_l <- abs(adjmeans_d[i]-adjmeans_d[j])
        Diff_u <- Diff_l + adjmeans_i[i] + adjmeans_i[j]

        Lower <- min(Diff_l,Diff_u)
        Upper <- max(Diff_l,Diff_u)

        if(Upper < LSD_l){

          Result <- "NS"

        }else if(Lower > LSD_u){

          Result <- "S"

        }else{

          Result <- "ID"

        }

        comparison <- rbind(
          comparison,
          data.frame(
            Treatment1=i,
            Treatment2=j,
            Difference=paste0("[",round(Lower,2),", ",round(Upper,2),"]"),
            LSD=paste0("[",round(LSD_l,2),", ",round(LSD_u,2),"]"),
            Result=Result,
            stringsAsFactors=FALSE
          )
        )

      }

    }

  }else{

    comparison <- NULL
    LSD_l <- NULL
    LSD_u <- NULL

  }

  #-------------------------------------------------------
  # Display Results
  #-------------------------------------------------------

  if(verbose){

    cat("\nNeutrosophic Analysis of Variance Table\n")
    print(nanova_table,row.names=FALSE)

    if(trt_significance != "NS"){

      cat("\nTreatment effect is significant. Hence multiple comparison using LSD is performed.\n")

      message("\nLSD Interval : [",
              round(LSD_l,4),
              ", ",
              round(LSD_u,4),
              "]")

      cat("\nTreatment Comparisons Using LSD\n")
      print(comparison,row.names=FALSE)

    }else{

      cat("\nTreatment effect is non significant. Hence multiple comparison is not performed.\n")

    }

    message("\nSignificance Codes:")
    message("*** : Significant at p < 0.001")
    message("**  : Significant at p < 0.01")
    message("*   : Significant at p < 0.05")
    message("NS  : Non Significant")
    message("ID  : Indeterminate")

  }

  #-------------------------------------------------------
  # Return Results
  #-------------------------------------------------------

  invisible(list(
    nanova_table = nanova_table,
    comparison = comparison,
    LSD = if(!is.null(LSD_l))
      c(Lower=LSD_l,Upper=LSD_u)
    else NULL
  ))
}

#' Neutrosophic Analysis of Covariance for RCBD
#'
#' Performs neutrosophic analysis of covariance for a Randomized Complete
#' Block Design using interval-valued response and covariate data.
#'
#' @param Lower_y Numeric matrix of lower response values.
#' @param Upper_y Numeric matrix of upper response values.
#' @param Lower_z Numeric matrix of lower covariate values.
#' @param Upper_z Numeric matrix of upper covariate values.
#' @param design Numeric matrix representing the RCBD layout.
#' @param alpha Significance level for LSD test. Default is `0.05`.
#' @param verbose Logical; if `TRUE`, prints the analysis. Default is `FALSE`.
#'
#' @return A list containing the NANCOVA table, LSD interval, and treatment
#' comparisons (if treatment effect is significant).
#'
#' @importFrom MASS ginv
#' @importFrom stats qf qt
#' @examples
#' Lower_y <- matrix(c(
#'   46.84, 97.69, 39.30, 49.20,
#'   51.52, 83.38, 56.48, 42.15,
#'   44.42, 49.64, 44.21, 35.79,
#'   32.77, 74.30, 55.72, 70.55
#' ), nrow = 4, byrow = TRUE)
#'
#' Upper_y <- matrix(c(
#'   49.66,101.31,47.70,55.80,
#'   60.98,87.62,60.52,44.85,
#'   52.26,59.36,52.79,44.61,
#'   42.23,82.70,59.28,75.45
#' ), nrow = 4, byrow = TRUE)
#'
#' Lower_z <- matrix(c(
#'   224.95,245.87,245.19,259.41,
#'   222.06,213.83,253.10,247.51,
#'   255.67,230.55,265.05,246.67,
#'   137.98,214.68,251.49,257.70
#' ), nrow = 4, byrow = TRUE)
#'
#' Upper_z <- matrix(c(
#'   229.05,250.13,252.81,268.59,
#'   229.94,222.17,258.90,256.49,
#'   262.33,237.45,274.95,249.33,
#'   142.02,219.32,260.51,264.30
#' ), nrow = 4, byrow = TRUE)
#'
#' design <- matrix(c(
#'   1,2,3,4,
#'   1,2,3,4,
#'   1,2,3,4,
#'   1,2,3,4
#' ), nrow = 4, byrow = TRUE)
#'
#' result <- RCBDnsANCOVA(
#'   Lower_y,
#'   Upper_y,
#'   Lower_z,
#'   Upper_z,
#'   design,
#'   alpha = 0.05,
#'   verbose = TRUE
#' )
#' @export


RCBDnsANCOVA <- function(Lower_y, Upper_y,
                         Lower_z, Upper_z,
                         design,
                         alpha = 0.05,
                         verbose = FALSE){

  #-------------------------------------------------------
  # Input Validation and Initialization
  #-------------------------------------------------------

  Lower_y <- as.matrix(Lower_y)
  Upper_y <- as.matrix(Upper_y)
  Lower_z <- as.matrix(Lower_z)
  Upper_z <- as.matrix(Upper_z)
  design  <- as.matrix(design)

  v <- max(design)
  b <- nrow(design)
  k <- ncol(design)

  rep_counts <- table(design)
  r <- unique(rep_counts)

  interval <- function(L, U){
    list(
      d = (L + U)/2,
      i = abs(U - L)/2
    )
  }

  n <- b * k

  #-------------------------------------------------------
  # Construction of Experimental Design and Model Matrices
  #-------------------------------------------------------

  trt_vec <- as.vector(t(design))

  Tmat <- matrix(0, n, v)

  for(i in 1:n){
    Tmat[i, trt_vec[i]] <- 1
  }

  mu <- matrix(1, n, 1)

  rep_vec <- rep(1:r, each = b/r * k)

  Rmat <- matrix(0, n, r)

  for(i in 1:n){
    Rmat[i, rep_vec[i]] <- 1
  }

  X <- cbind(Tmat, mu, Rmat)

  #-------------------------------------------------------
  # Interval Decomposition of Response and Covariate
  #-------------------------------------------------------

  d_y <- (Lower_y + Upper_y)/2
  i_y <- abs(Upper_y - Lower_y)/2

  d_z <- (Lower_z + Upper_z)/2
  i_z <- abs(Upper_z - Lower_z)/2

  #-------------------------------------------------------
  # Computation of Overall Interval Means and
  # Centered Interval Responses
  #-------------------------------------------------------

  ybar_l <- mean(Lower_y)
  ybar_u <- mean(Upper_y)
  ybar_d <- mean(d_y)
  ybar_i <- abs(ybar_u - ybar_l)/2

  zbar_l <- mean(Lower_z)
  zbar_u <- mean(Upper_z)
  zbar_d <- (zbar_l + zbar_u)/2
  zbar_i <- abs(zbar_u - zbar_l)/2

  yminusbar_l <- d_y - ybar_d
  yminusbar_u <- yminusbar_l + i_y - ybar_i

  yminusbar_d <- (yminusbar_l + yminusbar_u)/2
  yminusbar_i <- abs(yminusbar_u - yminusbar_l)/2

  zminusbar_l <- d_z - zbar_d
  zminusbar_u <- zminusbar_l + i_z - zbar_i

  zminusbar_d <- (zminusbar_l + zminusbar_u)/2
  zminusbar_i <- abs(zminusbar_u - zminusbar_l)/2

  dy <- as.vector(t(yminusbar_d))
  iy <- as.vector(t(yminusbar_i))

  dz <- as.vector(t(zminusbar_d))
  iz <- as.vector(t(zminusbar_i))

  #-------------------------------------------------------
  # Computation of Interval Cross Products
  #-------------------------------------------------------

  zpz_l <- t(dz) %*% dz

  zpz_u <- zpz_l +
    t(dz) %*% iz +
    t(iz) %*% dz +
    t(iz) %*% iz

  zpz_d <- (zpz_l + zpz_u)/2
  zpz_i <- abs(zpz_u - zpz_l)/2

  zpy_l <- t(dz) %*% dy

  zpy_u <- zpy_l +
    t(dz) %*% iy +
    t(iz) %*% dy +
    t(iz) %*% iy

  zpy_d <- (zpy_l + zpy_u)/2
  zpy_i <- abs(zpy_u - zpy_l)/2

  ypy_l <- t(dy) %*% dy

  ypy_u <- ypy_l +
    t(dy) %*% iy +
    t(iy) %*% dy +
    t(iy) %*% iy

  ypy_d <- (ypy_l + ypy_u)/2
  ypy_i <- abs(ypy_u - ypy_l)/2

  #-------------------------------------------------------
  # Estimation of Adjusted Treatment Effects
  #-------------------------------------------------------

  Z <- cbind(mu, Rmat)

  M <- diag(nrow(X)) - Z %*% MASS::ginv(t(Z) %*% Z) %*% t(Z)

  Ct_adj <- t(Tmat) %*% M %*% Tmat

  bcdy_t_ad_d <- t(Tmat) %*% cbind(mu, Rmat) %*%
    MASS::ginv(t(cbind(mu, Rmat)) %*% cbind(mu, Rmat)) %*%
    t(cbind(mu, Rmat)) %*% dy

  bcdy_t_ad_i <- t(Tmat) %*% cbind(mu, Rmat) %*%
    MASS::ginv(t(cbind(mu, Rmat)) %*% cbind(mu, Rmat)) %*%
    t(cbind(mu, Rmat)) %*% iy

  Qy_ad_l <- t(Tmat) %*% dy - bcdy_t_ad_d

  Qy_ad_u <- Qy_ad_l[(1:v), ] +
    t(Tmat) %*% iy - bcdy_t_ad_i

  Qy_t_ad_d <- (Qy_ad_l + Qy_ad_u)/2
  Qy_t_ad_i <- abs(Qy_ad_l - Qy_ad_u)/2

  ay_t_ad_l <- MASS::ginv(Ct_adj) %*% Qy_t_ad_d

  ay_t_ad_u <- ay_t_ad_l + Qy_t_ad_i/b

  ay_t_ad_d <- (ay_t_ad_l + ay_t_ad_u)/2
  ay_t_ad_i <- abs(ay_t_ad_l - ay_t_ad_u)/2

  #-------------------------------------------------------
  # Estimation of Adjusted Block Effects
  #-------------------------------------------------------

  Z <- cbind(mu, Tmat)

  M <- diag(nrow(X)) - Z %*% MASS::ginv(t(Z) %*% Z) %*% t(Z)

  Cb_adj <- t(Rmat) %*% M %*% Rmat

  bcdy_b_ad_d <- t(Rmat) %*% cbind(mu, Tmat) %*%
    MASS::ginv(t(cbind(mu, Tmat)) %*% cbind(mu, Tmat)) %*%
    t(cbind(mu, Tmat)) %*% dy

  bcdy_b_ad_i <- t(Rmat) %*% cbind(mu, Tmat) %*%
    MASS::ginv(t(cbind(mu, Tmat)) %*% cbind(mu, Tmat)) %*%
    t(cbind(mu, Tmat)) %*% iy

  Qy_ad_l <- t(Rmat) %*% dy - bcdy_b_ad_d

  Qy_ad_u <- Qy_ad_l[(1:b), ] +
    t(Rmat) %*% iy - bcdy_b_ad_i

  Qy_b_ad_d <- (Qy_ad_l + Qy_ad_u)/2
  Qy_b_ad_i <- abs(Qy_ad_l - Qy_ad_u)/2

  ay_b_ad_l <- MASS::ginv(Cb_adj) %*% Qy_b_ad_d

  ay_b_ad_u <- ay_b_ad_l + Qy_b_ad_i/v

  ay_b_ad_d <- (ay_b_ad_l + ay_b_ad_u)/2
  ay_b_ad_i <- abs(ay_b_ad_l - ay_b_ad_u)/2

  #-------------------------------------------------------
  # Estimation of Overall Mean Effects
  #-------------------------------------------------------

  Qy_mu_l <- sum(round(dy, 6))
  Qy_mu_u <- Qy_mu_l + sum(round(iy, 7))

  Qy_mu_d <- (Qy_mu_l + Qy_mu_u)/2
  Qy_mu_i <- abs(Qy_mu_l - Qy_mu_u)/2

  ay_mu_l <- Qy_mu_d/(b * k)
  ay_mu_u <- ay_mu_l + Qy_mu_d/(b * k)

  ay_mu_d <- (ay_mu_l + ay_mu_u)/2
  ay_mu_i <- abs(ay_mu_l - ay_mu_u)/2

  #-------------------------------------------------------
  # Combined Adjusted Effects
  #-------------------------------------------------------

  Qy_all_ad_d <- c(Qy_t_ad_d, Qy_mu_d, Qy_b_ad_d)
  Qy_all_ad_i <- c(Qy_t_ad_i, Qy_mu_i, Qy_b_ad_i)

  Qy_tmue_ad_d <- c(Qy_t_ad_d, Qy_mu_d)
  Qy_tmue_ad_i <- c(Qy_t_ad_i, Qy_mu_i)

  Qy_mueb_ad_d <- c(Qy_mu_d, Qy_b_ad_d)
  Qy_mueb_ad_i <- c(Qy_mu_i, Qy_b_ad_i)

  ay_all_ad_d <- c(ay_t_ad_d, ay_mu_d, ay_b_ad_d)
  ay_all_ad_i <- c(ay_t_ad_i, ay_mu_i, ay_b_ad_i)

  ay_tmue_ad_d <- c(ay_t_ad_d, ay_mu_d)
  ay_tmue_ad_i <- c(ay_t_ad_i, ay_mu_i)

  ay_mueb_ad_d <- c(ay_mu_d, ay_b_ad_d)
  ay_mueb_ad_i <- c(ay_mu_i, ay_b_ad_i)

  #-------------------------------------------------------
  # Estimation of Adjusted Covariate Effects
  #-------------------------------------------------------

  Z <- cbind(mu, Rmat)

  M <- diag(nrow(X)) - Z %*% MASS::ginv(t(Z) %*% Z) %*% t(Z)

  Ct_adj <- t(Tmat) %*% M %*% Tmat

  bcdz_t_ad_d <- t(Tmat) %*% cbind(mu, Rmat) %*%
    MASS::ginv(t(cbind(mu, Rmat)) %*% cbind(mu, Rmat)) %*%
    t(cbind(mu, Rmat)) %*% dz

  bcdz_t_ad_i <- t(Tmat) %*% cbind(mu, Rmat) %*%
    MASS::ginv(t(cbind(mu, Rmat)) %*% cbind(mu, Rmat)) %*%
    t(cbind(mu, Rmat)) %*% iz

  Qz_ad_l <- t(Tmat) %*% dz - bcdz_t_ad_d

  Qz_ad_u <- Qz_ad_l +
    t(Tmat) %*% iz -
    bcdz_t_ad_i

  Qz_t_ad_d <- (Qz_ad_l + Qz_ad_u)/2
  Qz_t_ad_i <- abs(Qz_ad_l - Qz_ad_u)/2

  az_t_ad_l <- MASS::ginv(Ct_adj) %*% Qz_t_ad_d

  az_t_ad_u <- az_t_ad_l + Qz_t_ad_i/r

  az_t_ad_d <- (az_t_ad_l + az_t_ad_u)/2
  az_t_ad_i <- abs(az_t_ad_l - az_t_ad_u)/2

  #-------------------------------------------------------
  # Estimation of Block Effects for Covariate
  #-------------------------------------------------------

  Cb_unadj <- t(Rmat) %*% Rmat

  bcdz_b_un_d <- t(cbind(Rmat, Tmat)) %*% mu %*%
    MASS::ginv(t(mu) %*% mu) %*%
    t(mu) %*% dz

  bcdz_b_un_i <- t(cbind(Rmat, Tmat)) %*% mu %*%
    MASS::ginv(t(mu) %*% mu) %*%
    t(mu) %*% iz

  Qz_un_l <- t(cbind(Rmat, Tmat)) %*% dz -
    bcdz_b_un_d

  Qz_un_u <- Qz_un_l +
    t(cbind(Rmat, Tmat)) %*% iz -
    bcdz_b_un_i

  Qz_b_un_d <- (Qz_un_l[1:r, ] + Qz_un_u[1:r, ])/2
  Qz_b_un_i <- abs(Qz_un_l[1:r, ] - Qz_un_u[1:r, ])/2

  az_b_l <- MASS::ginv(Cb_unadj) %*% Qz_b_un_d

  az_b_u <- az_b_l + Qz_b_un_i/v

  az_b_d <- (az_b_l + az_b_u)/2
  az_b_i <- abs(az_b_l - az_b_u)/2

  #-------------------------------------------------------
  # Estimation of Overall Mean for Covariate
  #-------------------------------------------------------

  Qz_mu_l <- sum(round(dz, 6))
  Qz_mu_u <- Qz_mu_l + sum(round(iz, 7))

  Qz_mu_d <- (Qz_mu_l + Qz_mu_u)/2
  Qz_mu_i <- abs(Qz_mu_l - Qz_mu_u)/2

  az_mu_l <- Qz_mu_d/(b * k)
  az_mu_u <- az_mu_l + Qz_mu_d/(b * k)

  az_mu_d <- (az_mu_l + az_mu_u)/2
  az_mu_i <- abs(az_mu_l - az_mu_u)/2

  #-------------------------------------------------------
  # Combined Adjusted Covariate Effects
  #-------------------------------------------------------

  Qz_all_ad_d <- c(Qz_t_ad_d, Qz_mu_d, Qz_b_un_d)
  Qz_all_ad_i <- c(Qz_t_ad_i, Qz_mu_i, Qz_b_un_i)

  Qz_tmue_ad_d <- c(Qz_t_ad_d, Qz_mu_d)
  Qz_tmue_ad_i <- c(Qz_t_ad_i, Qz_mu_i)

  Qz_mueb_ad_d <- c(Qz_mu_d, Qz_b_un_d)
  Qz_mueb_ad_i <- c(Qz_mu_i, Qz_b_un_i)

  az_all_ad_d <- c(az_t_ad_d, az_mu_d, az_b_d)
  az_all_ad_i <- c(az_t_ad_i, az_mu_i, az_b_i)

  az_tmue_ad_d <- c(az_t_ad_d, az_mu_d)
  az_tmue_ad_i <- c(az_t_ad_i, az_mu_i)

  az_mueb_ad_d <- c(az_mu_d, az_b_d)
  az_mueb_ad_i <- c(az_mu_i, az_b_i)

  #-------------------------------------------------------
  # Computation of QZAZ Products
  #-------------------------------------------------------

  QZAZ_all_ad_l <- c(
    Qz_t_ad_d * az_t_ad_d,
    Qz_mu_d * az_mu_d,
    Qz_b_un_d * az_b_d
  )

  QZAZ_all_ad_u <- QZAZ_all_ad_l +
    c(Qz_t_ad_d * az_t_ad_i,
      Qz_mu_d * az_mu_i,
      Qz_b_un_d * az_b_i) +
    c(Qz_t_ad_i * az_t_ad_d,
      Qz_mu_i * az_mu_d,
      Qz_b_un_i * az_b_d) +
    c(Qz_t_ad_i * az_t_ad_i,
      Qz_mu_i * az_mu_i,
      Qz_b_un_i * az_b_i)

  QZAZ_all_ad_d <- sum((QZAZ_all_ad_l + QZAZ_all_ad_u)/2)
  QZAZ_all_ad_i <- sum(abs(QZAZ_all_ad_u - QZAZ_all_ad_l)/2)

  QZAZ_tmue_ad_l <- c(
    Qz_t_ad_d * az_t_ad_d,
    Qz_mu_d * az_mu_d
  )

  QZAZ_tmue_ad_u <- QZAZ_tmue_ad_l +
    c(Qz_t_ad_d * az_t_ad_i,
      Qz_mu_d * az_mu_i) +
    c(Qz_t_ad_i * az_t_ad_d,
      Qz_mu_i * az_mu_d) +
    c(Qz_t_ad_i * az_t_ad_i,
      Qz_mu_i * az_mu_i)

  QZAZ_tmue_ad_d <- sum((QZAZ_tmue_ad_l + QZAZ_tmue_ad_u)/2)
  QZAZ_tmue_ad_i <- sum(abs((QZAZ_tmue_ad_u - QZAZ_tmue_ad_l)/2))

  QZAZ_mueb_ad_l <- c(
    Qz_mu_d * az_mu_d,
    Qz_b_un_d * az_b_d
  )

  QZAZ_mueb_ad_u <- QZAZ_mueb_ad_l +
    c(Qz_mu_d * az_mu_i,
      Qz_b_un_d * az_b_i) +
    c(Qz_mu_i * az_mu_d,
      Qz_b_un_i * az_b_d) +
    c(Qz_mu_i * az_mu_i,
      Qz_b_un_i * az_b_i)

  QZAZ_mueb_ad_d <- sum(QZAZ_mueb_ad_l + QZAZ_mueb_ad_u)/2
  QZAZ_mueb_ad_i <- sum(abs(QZAZ_mueb_ad_u - QZAZ_mueb_ad_l)/2)
  #-------------------------------------------------------
  # Computation of QZAy Products
  #-------------------------------------------------------

  QZAy_all_ad_l <- c(
    Qz_t_ad_d * ay_t_ad_d,
    Qz_mu_d * ay_mu_d,
    Qz_b_un_d * ay_b_ad_d
  )

  QZAy_all_ad_u <- QZAy_all_ad_l +
    c(Qz_t_ad_d * ay_t_ad_i,
      Qz_mu_d * ay_mu_i,
      Qz_b_un_d * ay_b_ad_i) +
    c(Qz_t_ad_i * ay_t_ad_d,
      Qz_mu_i * ay_mu_d,
      Qz_b_un_i * ay_b_ad_d) +
    c(Qz_t_ad_i * ay_t_ad_i,
      Qz_mu_i * ay_mu_i,
      Qz_b_un_i * ay_b_ad_i)

  QZAy_all_ad_d <- sum((QZAy_all_ad_l + QZAy_all_ad_u)/2)
  QZAy_all_ad_i <- sum(abs(QZAy_all_ad_u - QZAy_all_ad_l)/2)

  QZAy_tmue_ad_l <- c(
    Qz_t_ad_d * ay_t_ad_d,
    Qz_mu_d * ay_mu_d
  )

  QZAy_tmue_ad_u <- QZAy_tmue_ad_l +
    c(Qz_t_ad_d * ay_t_ad_i,
      Qz_mu_d * ay_mu_i) +
    c(Qz_t_ad_i * ay_t_ad_d,
      Qz_mu_i * ay_mu_d) +
    c(Qz_t_ad_i * ay_t_ad_i,
      Qz_mu_i * ay_mu_i)

  QZAy_tmue_ad_d <- sum((QZAy_tmue_ad_l + QZAy_tmue_ad_u)/2)
  QZAy_tmue_ad_i <- sum(abs((QZAy_tmue_ad_u - QZAy_tmue_ad_l)/2))

  QZAy_mueb_ad_l <- c(
    Qz_mu_d * ay_mu_d,
    Qz_b_un_d * ay_b_ad_d
  )

  QZAy_mueb_ad_u <- QZAy_mueb_ad_l +
    c(Qz_mu_d * ay_mu_i,
      Qz_b_un_d * ay_b_ad_i) +
    c(Qz_mu_i * ay_mu_d,
      Qz_b_un_i * ay_b_ad_d) +
    c(Qz_mu_i * ay_mu_i,
      Qz_b_un_i * ay_b_ad_i)

  QZAy_mueb_ad_d <- sum(QZAy_mueb_ad_l + QZAy_mueb_ad_u)/2
  QZAy_mueb_ad_i <- sum(abs(QZAy_mueb_ad_u - QZAy_mueb_ad_l)/2)

  #-------------------------------------------------------
  # Estimation of Regression Coefficients
  #-------------------------------------------------------

  EXX_all_ad_l <- zpz_d - QZAZ_all_ad_d
  EXX_all_ad_u <- EXX_all_ad_l + zpz_i - QZAZ_all_ad_i

  EXX_all_ad_d <- (EXX_all_ad_l + EXX_all_ad_u)/2
  EXX_all_ad_i <- abs(EXX_all_ad_u - EXX_all_ad_l)/2

  EXX_tmue_ad_l <- zpz_d - QZAZ_tmue_ad_d
  EXX_tmue_ad_u <- EXX_tmue_ad_l + zpz_i - QZAZ_tmue_ad_i

  EXX_tmue_ad_d <- (EXX_tmue_ad_l + EXX_tmue_ad_u)/2
  EXX_tmue_ad_i <- abs(EXX_tmue_ad_u - EXX_tmue_ad_l)/2

  EXX_mueb_ad_l <- zpz_d - QZAZ_mueb_ad_d
  EXX_mueb_ad_u <- EXX_mueb_ad_l + zpz_i - QZAZ_mueb_ad_i

  EXX_mueb_ad_d <- (EXX_mueb_ad_l + EXX_mueb_ad_u)/2
  EXX_mueb_ad_i <- abs(EXX_mueb_ad_u - EXX_mueb_ad_l)/2

  EXy_all_ad_l <- zpy_d - QZAy_all_ad_d
  EXy_all_ad_u <- EXy_all_ad_l + zpy_i - QZAy_all_ad_i

  EXy_all_ad_d <- (EXy_all_ad_l + EXy_all_ad_u)/2
  EXy_all_ad_i <- abs(EXy_all_ad_u - EXy_all_ad_l)/2

  EXy_tmue_ad_l <- zpy_d - QZAy_tmue_ad_d
  EXy_tmue_ad_u <- EXy_tmue_ad_l + zpy_i - QZAy_tmue_ad_i

  EXy_tmue_ad_d <- (EXy_tmue_ad_l + EXy_tmue_ad_u)/2
  EXy_tmue_ad_i <- abs(EXy_tmue_ad_u - EXy_tmue_ad_l)/2

  EXy_mueb_ad_l <- zpy_d - QZAy_mueb_ad_d
  EXy_mueb_ad_u <- EXy_mueb_ad_l + zpy_i - QZAy_mueb_ad_i

  EXy_mueb_ad_d <- (EXy_mueb_ad_l + EXy_mueb_ad_u)/2
  EXy_mueb_ad_i <- abs(EXy_mueb_ad_u - EXy_mueb_ad_l)/2

  gamma_1_ad_l <- EXy_all_ad_d/EXX_all_ad_d

  gamma_1_ad_u <- gamma_1_ad_l +
    (EXy_all_ad_d * EXX_all_ad_i +
       EXy_all_ad_i * EXX_all_ad_d +
       EXy_all_ad_i * EXX_all_ad_i) /
    (EXX_all_ad_d * EXX_all_ad_d)

  gamma_1_ad_d <- (gamma_1_ad_l + gamma_1_ad_u)/2
  gamma_1_ad_i <- abs(gamma_1_ad_u - gamma_1_ad_l)/2

  gamma_2_ad_l <- EXy_tmue_ad_d/EXX_tmue_ad_d

  gamma_2_ad_u <- gamma_2_ad_l +
    (EXy_tmue_ad_d * EXX_tmue_ad_i +
       EXy_tmue_ad_i * EXX_tmue_ad_d +
       EXy_tmue_ad_i * EXX_tmue_ad_i) /
    (EXX_tmue_ad_d * EXX_tmue_ad_d)

  gamma_2_ad_d <- (gamma_2_ad_l + gamma_2_ad_u)/2
  gamma_2_ad_i <- abs(gamma_2_ad_u - gamma_2_ad_l)/2

  gamma_3_ad_l <- EXy_mueb_ad_d/EXX_mueb_ad_d

  gamma_3_ad_u <- gamma_3_ad_l +
    (EXy_mueb_ad_d * EXX_mueb_ad_i +
       EXy_mueb_ad_i * EXX_mueb_ad_d +
       EXy_mueb_ad_i * EXX_mueb_ad_i) /
    (EXX_mueb_ad_d * EXX_mueb_ad_d)

  gamma_3_ad_d <- (gamma_3_ad_l + gamma_3_ad_u)/2
  gamma_3_ad_i <- abs(gamma_3_ad_u - gamma_3_ad_l)/2
  #-------------------------------------------------------
  # Estimation of Adjusted Responses
  #-------------------------------------------------------

  gaz_all_ad_l <- az_all_ad_d %*% gamma_1_ad_d

  gaz_all_ad_u <- gaz_all_ad_l +
    az_all_ad_d %*% gamma_1_ad_i +
    az_all_ad_i %*% gamma_1_ad_d +
    az_all_ad_i %*% gamma_1_ad_i

  gaz_all_ad_d <- (gaz_all_ad_l + gaz_all_ad_u)/2
  gaz_all_ad_i <- abs(gaz_all_ad_u - gaz_all_ad_l)/2

  acap_all_ad_l <- ay_all_ad_d - gaz_all_ad_d
  acap_all_ad_u <- acap_all_ad_l + ay_all_ad_i - gaz_all_ad_i

  acap_all_ad_d <- (acap_all_ad_l + acap_all_ad_u)/2
  acap_all_ad_i <- abs(acap_all_ad_u - acap_all_ad_l)/2


  gaz_tmue_ad_l <- az_tmue_ad_d %*% gamma_2_ad_d

  gaz_tmue_ad_u <- gaz_tmue_ad_l +
    az_tmue_ad_d %*% gamma_2_ad_i +
    az_tmue_ad_i %*% gamma_2_ad_d +
    az_tmue_ad_i %*% gamma_2_ad_i

  gaz_tmue_ad_d <- (gaz_tmue_ad_l + gaz_tmue_ad_u)/2
  gaz_tmue_ad_i <- abs(gaz_tmue_ad_u - gaz_tmue_ad_l)/2

  acap_tmue_ad_l <- ay_tmue_ad_d - gaz_tmue_ad_d
  acap_tmue_ad_u <- acap_tmue_ad_l + ay_tmue_ad_i - gaz_tmue_ad_i

  acap_tmue_ad_d <- (acap_tmue_ad_l + acap_tmue_ad_u)/2
  acap_tmue_ad_i <- abs(acap_tmue_ad_u - acap_tmue_ad_l)/2


  gaz_mueb_ad_l <- az_mueb_ad_d %*% gamma_3_ad_d

  gaz_mueb_ad_u <- gaz_mueb_ad_l +
    az_mueb_ad_d %*% gamma_3_ad_i +
    az_mueb_ad_i %*% gamma_3_ad_d +
    az_mueb_ad_i %*% gamma_3_ad_i

  gaz_mueb_ad_d <- (gaz_mueb_ad_l + gaz_mueb_ad_u)/2
  gaz_mueb_ad_i <- abs(gaz_mueb_ad_u - gaz_mueb_ad_l)/2

  acap_mueb_ad_l <- ay_mueb_ad_d - gaz_mueb_ad_d
  acap_mueb_ad_u <- acap_mueb_ad_l + ay_mueb_ad_i - gaz_mueb_ad_i

  acap_mueb_ad_d <- (acap_mueb_ad_l + acap_mueb_ad_u)/2
  acap_mueb_ad_i <- abs(acap_mueb_ad_u - acap_mueb_ad_l)/2

  #-------------------------------------------------------
  # Computation of Adjusted Sum of Squares
  #-------------------------------------------------------

  acapss_all_ad_l <- Qy_all_ad_d * acap_all_ad_d

  acapss_all_ad_u <- acapss_all_ad_l +
    Qy_all_ad_d * acap_all_ad_i +
    Qy_all_ad_i * acap_all_ad_d +
    Qy_all_ad_i * acap_all_ad_i

  acapss_all_ad_d <- (acapss_all_ad_u + acapss_all_ad_l)/2
  acapss_all_ad_i <- abs(acapss_all_ad_u - acapss_all_ad_l)/2


  acapss_tmue_ad_l <- Qy_tmue_ad_d * acap_tmue_ad_d

  acapss_tmue_ad_u <- acapss_tmue_ad_l +
    Qy_tmue_ad_d * acap_tmue_ad_i +
    Qy_tmue_ad_i * acap_tmue_ad_d +
    Qy_tmue_ad_i * acap_tmue_ad_i

  acapss_tmue_ad_d <- (acapss_tmue_ad_u + acapss_tmue_ad_l)/2
  acapss_tmue_ad_i <- abs(acapss_tmue_ad_u - acapss_tmue_ad_l)/2


  acapss_mueb_ad_l <- Qy_mueb_ad_d * acap_mueb_ad_d

  acapss_mueb_ad_u <- acapss_mueb_ad_l +
    Qy_mueb_ad_d * acap_mueb_ad_i +
    Qy_mueb_ad_i * acap_mueb_ad_d +
    Qy_mueb_ad_i * acap_mueb_ad_i

  acapss_mueb_ad_d <- (acapss_mueb_ad_u + acapss_mueb_ad_l)/2
  acapss_mueb_ad_i <- abs(acapss_mueb_ad_u - acapss_mueb_ad_l)/2


  ss_all_ad_l <- sum(acapss_all_ad_l)
  ss_all_ad_u <- sum(acapss_all_ad_u)
  ss_all_ad_d <- (ss_all_ad_l + ss_all_ad_u)/2
  ss_all_ad_i <- sum(acapss_all_ad_i)

  ss_tmue_ad_l <- sum(acapss_tmue_ad_l)
  ss_tmue_ad_u <- sum(acapss_tmue_ad_u)
  ss_tmue_ad_d <- (ss_tmue_ad_l + ss_tmue_ad_u)/2
  ss_tmue_ad_i <- sum(acapss_tmue_ad_i)

  ss_mueb_ad_l <- sum(acapss_mueb_ad_l)
  ss_mueb_ad_u <- sum(acapss_mueb_ad_u)
  ss_mueb_ad_d <- sum(acapss_mueb_ad_d)
  ss_mueb_ad_i <- sum(acapss_mueb_ad_i)

  #-------------------------------------------------------
  # Computation of Regression Sum of Squares
  #-------------------------------------------------------

  gzy_1_ad_l <- gamma_1_ad_d * zpy_d

  gzy_1_ad_u <- gzy_1_ad_l +
    gamma_1_ad_d * zpy_i +
    gamma_1_ad_i * zpy_d +
    gamma_1_ad_i * zpy_i

  gzy_1_ad_d <- (gzy_1_ad_l + gzy_1_ad_u)/2
  gzy_1_ad_i <- abs(gzy_1_ad_u - gzy_1_ad_l)/2


  gzy_2_ad_l <- gamma_2_ad_d * zpy_d

  gzy_2_ad_u <- gzy_2_ad_l +
    gamma_2_ad_d * zpy_i +
    gamma_2_ad_i * zpy_d +
    gamma_2_ad_i * zpy_i

  gzy_2_ad_d <- (gzy_2_ad_l + gzy_2_ad_u)/2
  gzy_2_ad_i <- abs(gzy_2_ad_u - gzy_2_ad_l)/2


  gzy_3_ad_l <- gamma_3_ad_d * zpy_d

  gzy_3_ad_u <- gzy_3_ad_l +
    gamma_3_ad_d * zpy_i +
    gamma_3_ad_i * zpy_d +
    gamma_3_ad_i * zpy_i

  gzy_3_ad_d <- (gzy_3_ad_l + gzy_3_ad_u)/2
  gzy_3_ad_i <- abs(gzy_3_ad_u - gzy_3_ad_l)/2

  #-------------------------------------------------------
  # Computation of Model Sum of Squares
  #-------------------------------------------------------

  mdss_all_ad_l <- ss_all_ad_d + gzy_1_ad_d
  mdss_all_ad_u <- mdss_all_ad_l + ss_all_ad_i + gzy_1_ad_i

  mdss_all_ad_d <- (mdss_all_ad_u + mdss_all_ad_l)/2
  mdss_all_ad_i <- abs(mdss_all_ad_u - mdss_all_ad_l)/2


  mdss_tmue_ad_l <- ss_tmue_ad_d + gzy_2_ad_d
  mdss_tmue_ad_u <- mdss_tmue_ad_l + ss_tmue_ad_i + gzy_2_ad_i

  mdss_tmue_ad_d <- (mdss_tmue_ad_u + mdss_tmue_ad_l)/2
  mdss_tmue_ad_i <- abs(mdss_tmue_ad_u - mdss_tmue_ad_l)/2


  mdss_mueb_ad_l <- ss_mueb_ad_d + gzy_3_ad_d
  mdss_mueb_ad_u <- mdss_mueb_ad_l + ss_mueb_ad_i + gzy_3_ad_i

  mdss_mueb_ad_d <- (mdss_mueb_ad_u + mdss_mueb_ad_l)/2
  mdss_mueb_ad_i <- abs(mdss_mueb_ad_u - mdss_mueb_ad_l)/2
  #-------------------------------------------------------
  # Computation of Error Sum of Squares
  #-------------------------------------------------------

  ess_all_ad_l <- ypy_d - mdss_all_ad_d

  ess_all_ad_u <- ess_all_ad_l +
    ypy_i - mdss_all_ad_i

  ess_all_ad_d <- (ess_all_ad_l + ess_all_ad_u)/2
  ess_all_ad_i <- abs(ess_all_ad_u - ess_all_ad_l)/2


  esst_ad_l <- ypy_d - mdss_mueb_ad_d

  esst_ad_u <- esst_ad_l +
    ypy_i - mdss_mueb_ad_i

  esst_ad_d <- (esst_ad_u + esst_ad_l)/2
  esst_ad_i <- abs(esst_ad_u - esst_ad_l)/2


  essb_ad_l <- ypy_d - mdss_tmue_ad_d

  essb_ad_u <- essb_ad_l +
    ypy_i - mdss_tmue_ad_i

  essb_ad_d <- (essb_ad_u + essb_ad_l)/2
  essb_ad_i <- abs(essb_ad_u - essb_ad_l)/2

  #-------------------------------------------------------
  # Computation of Adjusted Treatment and
  # Replication Sum of Squares
  #-------------------------------------------------------

  tss_ad_l <- esst_ad_d - ess_all_ad_d

  tss_ad_u <- tss_ad_l +
    esst_ad_i - ess_all_ad_i

  tss_ad_d <- (tss_ad_u + tss_ad_l)/2
  tss_ad_i <- abs(tss_ad_u - tss_ad_l)/2


  bss_ad_l <- essb_ad_d - ess_all_ad_d

  bss_ad_u <- bss_ad_l +
    essb_ad_i - ess_all_ad_i

  bss_ad_d <- (bss_ad_u + bss_ad_l)/2
  bss_ad_i <- abs(bss_ad_u - bss_ad_l)/2

  #-------------------------------------------------------
  # Computation of Covariate Sum of Squares
  #-------------------------------------------------------

  gss_l <- gamma_1_ad_d * EXy_all_ad_d

  gss_u <- gss_l +
    gamma_1_ad_d * EXy_all_ad_i +
    gamma_1_ad_i * EXy_all_ad_d +
    gamma_1_ad_i * EXy_all_ad_i

  gss_d <- (gss_l + gss_u)/2
  gss_i <- abs(gss_u - gss_l)/2

  #-------------------------------------------------------
  # Computation of Total Sum of Squares
  #-------------------------------------------------------

  toss_l <- t(dy) %*% dy

  toss_u <- toss_l +
    2 * t(dy) %*% iy +
    t(iy) %*% iy

  toss_d <- (toss_l + toss_u)/2
  toss_i <- abs(toss_u - toss_l)/2
  #-------------------------------------------------------
  # Computation of Interval Mean Squares and F-Statistics
  #-------------------------------------------------------

  SV <- c(
    "Covariate",
    "Treatment",
    "Replication",
    "Error",
    "Total"
  )

  DF <- c(
    1,
    v - 1,
    b - 1,
    (v - 1) * (b - 1) - 1,
    n - 1
  )

  SSN <- list(
    c(gss_l, gss_u),
    c(tss_ad_l, tss_ad_u),
    c(bss_ad_l, bss_ad_u),
    c(ess_all_ad_l, ess_all_ad_u),
    c(toss_l, toss_u)
  )

  MSSN <- list(
    c(gss_l, gss_u),
    c(tss_ad_l/(v - 1), tss_ad_u/(v - 1)),
    c(bss_ad_l/(b - 1), bss_ad_u/(b - 1)),
    c(
      ess_all_ad_l/((v - 1) * (b - 1) - 1),
      ess_all_ad_u/((v - 1) * (b - 1) - 1)
    ),
    c(NA, NA)
  )

  F_g_L <- (gss_d/1) /
    (ess_all_ad_d/((v - 1) * (b - 1) - 1))

  F_g_U <- F_g_L +
    (
      (gss_d/1) *
        (ess_all_ad_i/((v - 1) * (b - 1) - 1)) +
        (gss_i/1) *
        (ess_all_ad_d/((v - 1) * (b - 1) - 1)) +
        (gss_i/1) *
        (ess_all_ad_i/((v - 1) * (b - 1) - 1))
    ) /
    (
      (ess_all_ad_d/((v - 1) * (b - 1) - 1)) *
        (ess_all_ad_d/((v - 1) * (b - 1) - 1))
    )

  F_t_ad_L <- (tss_ad_d/(v - 1)) /
    (ess_all_ad_d/((v - 1) * (b - 1) - 1))

  F_t_ad_U <- F_t_ad_L +
    (
      ((tss_ad_d/(v - 1)) *
         (ess_all_ad_i/((v - 1) * (b - 1) - 1))) +
        ((tss_ad_i/(v - 1)) *
           (ess_all_ad_d/((v - 1) * (b - 1) - 1))) +
        ((tss_ad_i/(v - 1)) *
           (ess_all_ad_i/((v - 1) * (b - 1) - 1)))
    ) /
    (
      (ess_all_ad_d/((v - 1) * (b - 1) - 1)) *
        (ess_all_ad_d/((v - 1) * (b - 1) - 1))
    )

  F_b_ad_L <- (bss_ad_d/(b - 1)) /
    (ess_all_ad_d/((v - 1) * (b - 1) - 1))

  F_b_ad_U <- F_b_ad_L +
    (
      ((bss_ad_d/(b - 1)) *
         (ess_all_ad_i/((v - 1) * (b - 1) - 1))) +
        ((bss_ad_i/(b - 1)) *
           (ess_all_ad_d/((v - 1) * (b - 1) - 1))) +
        ((bss_ad_i/(b - 1)) *
           (ess_all_ad_i/((v - 1) * (b - 1) - 1)))
    ) /
    (
      (ess_all_ad_d/((v - 1) * (b - 1) - 1)) *
        (ess_all_ad_d/((v - 1) * (b - 1) - 1))
    )

  FN <- list(
    c(F_g_L, F_g_U),
    c(F_t_ad_L, F_t_ad_U),
    c(F_b_ad_L, F_b_ad_U),
    c(NA, NA),
    c(NA, NA)
  )

  format_interval <- function(x){
    if(any(is.na(x))) return("")
    paste0("[", round(x[1], 2), ", ", round(x[2], 2), "]")
  }

  SSN_fmt  <- sapply(SSN, format_interval)
  MSSN_fmt <- sapply(MSSN, format_interval)
  FN_fmt   <- sapply(FN, format_interval)

  nancova_table <- data.frame(
    SV = SV,
    DF = DF,
    SSN = SSN_fmt,
    MSSN = MSSN_fmt,
    FN = FN_fmt,
    stringsAsFactors = FALSE
  )

  #-------------------------------------------------------
  # Determination of Significance
  #-------------------------------------------------------

  Ftab5_cov <- qf(0.95, 1, (v-1)*(b-1)-1)
  Ftab1_cov <- qf(0.99, 1, (v-1)*(b-1)-1)
  Ftab001_cov <- qf(0.999, 1, (v-1)*(b-1)-1)

  Ftab5_trt <- qf(0.95, v-1, (v-1)*(b-1)-1)
  Ftab1_trt <- qf(0.99, v-1, (v-1)*(b-1)-1)
  Ftab001_trt <- qf(0.999, v-1, (v-1)*(b-1)-1)

  Ftab5_blk <- qf(0.95, b-1, (v-1)*(b-1)-1)
  Ftab1_blk <- qf(0.99, b-1, (v-1)*(b-1)-1)
  Ftab001_blk <- qf(0.999, b-1, (v-1)*(b-1)-1)

  sig_cov <- ifelse(
    F_g_L > Ftab001_cov,"***",
    ifelse(F_g_L > Ftab1_cov,"**",
           ifelse(F_g_L > Ftab5_cov,"*","NS"))
  )

  sig_trt <- ifelse(
    F_t_ad_L > Ftab001_trt,"***",
    ifelse(F_t_ad_L > Ftab1_trt,"**",
           ifelse(F_t_ad_L > Ftab5_trt,"*","NS"))
  )

  sig_blk <- ifelse(
    F_b_ad_L > Ftab001_blk,"***",
    ifelse(F_b_ad_L > Ftab1_blk,"**",
           ifelse(F_b_ad_L > Ftab5_blk,"*","NS"))
  )

  nancova_table$Significance <- c(
    sig_cov,
    sig_trt,
    sig_blk,
    "",
    ""
  )

  #-------------------------------------------------------
  # Interval-Based LSD Analysis
  #-------------------------------------------------------

  dfe <- (v-1)*(b-1)-1

  MSE_d <- ess_all_ad_d/dfe
  MSE_i <- ess_all_ad_i/dfe

  SEd_d <- sqrt((2*MSE_d)/r)
  SEd_i <- sqrt((2*MSE_i)/r)

  tval <- qt(1-alpha/2,dfe)

  LSD_l <- tval*SEd_d
  LSD_u <- LSD_l+tval*SEd_i

  trtmeans_d <- as.vector(acap_all_ad_d[1:v])
  trtmeans_i <- as.vector(acap_all_ad_i[1:v])

  grand_mean <- mean((Lower_y+Upper_y)/2)

  adjmeans_d <- grand_mean+trtmeans_d
  adjmeans_i <- trtmeans_i

  comparison <- data.frame()

  for(i in 1:(v-1)){

    for(j in (i+1):v){

      Diff_l <- abs(adjmeans_d[i]-adjmeans_d[j])

      Diff_u <- Diff_l+
        adjmeans_i[i]+
        adjmeans_i[j]

      Lower <- min(Diff_l,Diff_u)
      Upper <- max(Diff_l,Diff_u)

      if(Upper<LSD_l){

        result <- "NS"

      }else if(Lower>LSD_u){

        result <- "S"

      }else{

        result <- "I"

      }

      comparison <- rbind(
        comparison,
        data.frame(
          Treatment1=i,
          Treatment2=j,
          Difference=paste0("[",round(Lower,2),", ",round(Upper,2),"]"),
          LSD=paste0("[",round(LSD_l,2),", ",round(LSD_u,2),"]"),
          Result=result
        )
      )

    }

  }

  #-------------------------------------------------------
  # Display Results
  #-------------------------------------------------------

  if(verbose){

    message("\nNeutrosophic Analysis of Covariance Table")

    print(nancova_table,row.names=FALSE)

    if(sig_trt!="NS"){

      message(
        "\nLSD Interval : [",
        round(LSD_l,4),
        ", ",
        round(LSD_u,4),
        "]"
      )

      message("\nTreatment Comparisons Using LSD")

      print(comparison,row.names=FALSE)

    }else{

      message("\nTreatment effect is non-significant.")
      message("Multiple comparison is not performed.")

    }

    message("\nSignificance Codes:")
    message("*** : Significant at p < 0.001")
    message("**  : Significant at p < 0.01")
    message("*   : Significant at p < 0.05")
    message("NS  : Non Significant")
    message("I   : Indeterminate")
    #-------------------------------------------------------
    # Return Results
    #-------------------------------------------------------

    invisible(list(

      nancova_table=nancova_table,

      comparison=if(sig_trt!="NS")
        comparison
      else
        NULL,

      LSD=if(sig_trt!="NS")
        c(
          Lower=LSD_l,
          Upper=LSD_u
        )
      else
        NULL

    ))
  }
}

#' Neutrosophic Analysis of Covariance for Completely Randomized Design
#'
#' Performs Neutrosophic Analysis of Covariance (NANCOVA) for interval-valued
#' response and covariate data from a Completely Randomized Design (CRD).
#'
#' @param Lower_y Numeric matrix of lower bounds of the response variable.
#' @param Upper_y Numeric matrix of upper bounds of the response variable.
#' @param Lower_z Numeric matrix of lower bounds of the covariate.
#' @param Upper_z Numeric matrix of upper bounds of the covariate.
#' @param design Numeric matrix representing the CRD treatment layout.
#' @param alpha Significance level for interval-based LSD comparisons. Default is 0.05.
#' @param verbose Logical. If \code{TRUE}, prints the NANCOVA table and LSD comparisons.
#'
#' @return A list containing the Neutrosophic ANCOVA table, interval LSD comparisons
#' (if treatment effects are significant), and the interval LSD value.
#'
#' @examples
#' Lower_y <- matrix(c(
#'   20,25,30,35,
#'   21,26,31,36,
#'   19,24,29,34,
#'   20,25,30,35,
#'   21,26,31,36
#' ), nrow = 5, byrow = TRUE)
#'
#' Upper_y <- matrix(c(
#'   22,27,32,37,
#'   23,28,33,38,
#'   21,26,31,36,
#'   22,27,32,37,
#'   23,28,33,38
#' ), nrow = 5, byrow = TRUE)
#'
#' Lower_z <- matrix(c(
#'   8,9,10,11,
#'   9,10,11,12,
#'   8,9,10,11,
#'   9,10,11,12,
#'   8,9,10,11
#' ), nrow = 5, byrow = TRUE)
#'
#' Upper_z <- matrix(c(
#'   9,10,11,12,
#'   10,11,12,13,
#'   9,10,11,12,
#'   10,11,12,13,
#'   9,10,11,12
#' ), nrow = 5, byrow = TRUE)
#'
#' design <- matrix(c(
#'   1,2,3,4,
#'   1,2,3,4,
#'   1,2,3,4,
#'   1,2,3,4,
#'   1,2,3,4
#' ), nrow = 5, byrow = TRUE)
#'
#' CRDnsANCOVA(Lower_y, Upper_y, Lower_z, Upper_z, design)
#'
#' @importFrom MASS ginv
#' @importFrom stats qf qt
#' @export

CRDnsANCOVA <- function(Lower_y, Upper_y,
                        Lower_z, Upper_z,
                        design,
                        alpha = 0.05,
                        verbose = FALSE){

  #-------------------------------------------------------
  # Input Validation and Initialization
  #-------------------------------------------------------

  Lower_y <- as.matrix(Lower_y)
  Upper_y <- as.matrix(Upper_y)
  Lower_z <- as.matrix(Lower_z)
  Upper_z <- as.matrix(Upper_z)
  design  <- as.matrix(design)

  v <- max(design)
  b <- nrow(design)
  k <- ncol(design)

  rep_counts <- table(design)
  r <- unique(rep_counts)

  interval <- function(L, U){
    list(
      d = (L + U)/2,
      i = abs(U - L)/2
    )
  }

  n <- b * k

  #-------------------------------------------------------
  # Construction of Experimental Design and Model Matrices
  #-------------------------------------------------------

  trt_vec <- as.vector(t(design))

  Tmat <- matrix(0, n, v)

  for(i in 1:n){
    Tmat[i, trt_vec[i]] <- 1
  }

  mu <- matrix(1, n, 1)

  X <- cbind(Tmat, mu)

  #-------------------------------------------------------
  # Interval Decomposition of Responses and Covariates
  #-------------------------------------------------------

  d_y <- (Lower_y + Upper_y)/2
  i_y <- abs(Upper_y - Lower_y)/2

  d_z <- (Lower_z + Upper_z)/2
  i_z <- abs(Upper_z - Lower_z)/2

  #-------------------------------------------------------
  # Computation of Overall Interval Means and
  # Centered Responses and Covariates
  #-------------------------------------------------------

  ybar_l <- mean(Lower_y)
  ybar_u <- mean(Upper_y)
  ybar_d <- mean(d_y)
  ybar_i <- abs(ybar_u - ybar_l)/2

  zbar_l <- mean(Lower_z)
  zbar_u <- mean(Upper_z)
  zbar_d <- (zbar_l + zbar_u)/2
  zbar_i <- abs(zbar_u - zbar_l)/2

  yminusbar_l <- d_y - ybar_d
  yminusbar_u <- yminusbar_l + i_y - ybar_i
  yminusbar_d <- (yminusbar_l + yminusbar_u)/2
  yminusbar_i <- abs(yminusbar_u - yminusbar_l)/2

  zminusbar_l <- d_z - zbar_d
  zminusbar_u <- zminusbar_l + i_z - zbar_i
  zminusbar_d <- (zminusbar_l + zminusbar_u)/2
  zminusbar_i <- abs(zminusbar_u - zminusbar_l)/2

  dy <- as.vector(t(yminusbar_d))
  iy <- as.vector(t(yminusbar_i))

  dz <- as.vector(t(zminusbar_d))
  iz <- as.vector(t(zminusbar_i))

  #-------------------------------------------------------
  # Computation of Interval Cross Products
  #-------------------------------------------------------

  zpz_l <- t(dz) %*% dz
  zpz_u <- zpz_l +
    t(dz) %*% iz +
    t(iz) %*% dz +
    t(iz) %*% iz

  zpz_d <- (zpz_l + zpz_u)/2
  zpz_i <- abs(zpz_u - zpz_l)/2

  zpy_l <- t(dz) %*% dy
  zpy_u <- zpy_l +
    t(dz) %*% iy +
    t(iz) %*% dy +
    t(iz) %*% iy

  zpy_d <- (zpy_l + zpy_u)/2
  zpy_i <- abs(zpy_u - zpy_l)/2

  ypy_l <- t(dy) %*% dy
  ypy_u <- ypy_l +
    t(dy) %*% iy +
    t(iy) %*% dy +
    t(iy) %*% iy

  ypy_d <- (ypy_l + ypy_u)/2
  ypy_i <- abs(ypy_u - ypy_l)/2


  #-------------------------------------------------------
  # Estimation of Adjusted Treatment and Overall Mean Effects
  #-------------------------------------------------------

  Z <- cbind(mu)

  M <- diag(nrow(X)) - Z %*% ginv(t(Z) %*% Z) %*% t(Z)

  Ct_adj <- t(Tmat) %*% M %*% Tmat

  bcdy_t_ad_d <- t(Tmat) %*% cbind(mu) %*%
    ginv(t(cbind(mu)) %*% cbind(mu)) %*%
    t(cbind(mu)) %*% dy

  bcdy_t_ad_i <- t(Tmat) %*% cbind(mu) %*%
    ginv(t(cbind(mu)) %*% cbind(mu)) %*%
    t(cbind(mu)) %*% iy

  Qy_ad_l <- t(Tmat) %*% dy - bcdy_t_ad_d

  Qy_ad_u <- Qy_ad_l[(1:v), ] +
    t(Tmat) %*% iy - bcdy_t_ad_i

  Qy_t_ad_d <- (Qy_ad_l + Qy_ad_u)/2
  Qy_t_ad_i <- abs(Qy_ad_l - Qy_ad_u)/2

  ay_t_ad_l <- ginv(Ct_adj) %*% Qy_t_ad_d

  ay_t_ad_u <- ay_t_ad_l + Qy_t_ad_i/b

  ay_t_ad_d <- (ay_t_ad_l + ay_t_ad_u)/2
  ay_t_ad_i <- abs(ay_t_ad_l - ay_t_ad_u)/2

  ################ Overall Mean ################

  Qy_mu_l <- sum(round(dy, 6))
  Qy_mu_u <- Qy_mu_l + sum(round(iy, 7))

  Qy_mu_d <- (Qy_mu_l + Qy_mu_u)/2
  Qy_mu_i <- abs(Qy_mu_l - Qy_mu_u)/2

  ay_mu_l <- Qy_mu_d/(b * k)
  ay_mu_u <- ay_mu_l + Qy_mu_d/(b * k)

  ay_mu_d <- (ay_mu_l + ay_mu_u)/2
  ay_mu_i <- abs(ay_mu_l - ay_mu_u)/2

  Qy_all_ad_d <- c(Qy_t_ad_d, Qy_mu_d)
  Qy_all_ad_i <- c(Qy_t_ad_i, Qy_mu_i)

  Qy_t_ad_d <- c(Qy_t_ad_d)
  Qy_t_ad_i <- c(Qy_t_ad_i)

  Qy_mue_ad_d <- c(Qy_mu_d)
  Qy_mue_ad_i <- c(Qy_mu_i)

  ay_all_ad_d <- c(ay_t_ad_d, ay_mu_d)
  ay_all_ad_i <- c(ay_t_ad_i, ay_mu_i)

  ay_t_ad_d <- c(ay_t_ad_d)
  ay_t_ad_i <- c(ay_t_ad_i)

  ay_mue_ad_d <- c(ay_mu_d)
  ay_mue_ad_i <- c(ay_mu_i)


  #-------------------------------------------------------
  # Estimation of Adjusted Covariate and Overall Mean Effects
  #-------------------------------------------------------

  Z <- cbind(mu)

  M <- diag(nrow(X)) - Z %*% ginv(t(Z) %*% Z) %*% t(Z)

  Ct_adj <- t(Tmat) %*% M %*% Tmat

  bcdz_t_ad_d <- t(Tmat) %*% cbind(mu) %*%
    ginv(t(cbind(mu)) %*% cbind(mu)) %*%
    t(cbind(mu)) %*% dz

  bcdz_t_ad_i <- t(Tmat) %*% cbind(mu) %*%
    ginv(t(cbind(mu)) %*% cbind(mu)) %*%
    t(cbind(mu)) %*% iz

  #### Covariate

  Z <- cbind(mu)

  M <- diag(nrow(X)) - Z %*% ginv(t(Z) %*% Z) %*% t(Z)

  Ct_adj <- t(Tmat) %*% M %*% Tmat

  bcdz_t_ad_d <- t(Tmat) %*% cbind(mu) %*%
    ginv(t(cbind(mu)) %*% cbind(mu)) %*%
    t(cbind(mu)) %*% dz

  bcdz_t_ad_i <- t(Tmat) %*% cbind(mu) %*%
    ginv(t(cbind(mu)) %*% cbind(mu)) %*%
    t(cbind(mu)) %*% iz

  Qz_ad_l <- t(Tmat) %*% dz - bcdz_t_ad_d

  Qz_ad_u <- Qz_ad_l +
    t(Tmat) %*% iz - bcdz_t_ad_i

  Qz_t_ad_d <- (Qz_ad_l + Qz_ad_u)/2
  Qz_t_ad_i <- abs(Qz_ad_l - Qz_ad_u)/2

  az_t_ad_l <- ginv(Ct_adj) %*% Qz_t_ad_d

  az_t_ad_u <- az_t_ad_l + Qz_t_ad_i/r

  az_t_ad_d <- (az_t_ad_l + az_t_ad_u)/2
  az_t_ad_i <- abs(az_t_ad_l - az_t_ad_u)/2

  ################ Overall Mean ################

  Qz_mu_l <- sum(round(dz, 6))
  Qz_mu_u <- Qz_mu_l + sum(round(iz, 7))

  Qz_mu_d <- (Qz_mu_l + Qz_mu_u)/2
  Qz_mu_i <- abs(Qz_mu_l - Qz_mu_u)/2

  az_mu_l <- Qz_mu_d/(b * k)
  az_mu_u <- az_mu_l + Qz_mu_d/(b * k)

  az_mu_d <- (az_mu_l + az_mu_u)/2
  az_mu_i <- abs(az_mu_l - az_mu_u)/2

  Qz_all_ad_d <- c(Qz_t_ad_d, Qz_mu_d)
  Qz_all_ad_i <- c(Qz_t_ad_i, Qz_mu_i)

  Qz_mue_ad_d <- c(Qz_mu_d)
  Qz_mue_ad_i <- c(Qz_mu_i)

  az_all_ad_d <- c(az_t_ad_d, az_mu_d)
  az_all_ad_i <- c(az_t_ad_i, az_mu_i)

  az_mue_ad_d <- c(az_mu_d)
  az_mue_ad_i <- c(az_mu_i)

  #-------------------------------------------------------
  # Computation of Interval Cross Products for
  # Regression Coefficient Estimation
  #-------------------------------------------------------

  #### Qz × Az

  QZAZ_all_ad_l <- c(Qz_t_ad_d * az_t_ad_d,
                     Qz_mu_d * az_mu_d)

  QZAZ_all_ad_u <- QZAZ_all_ad_l +
    c(Qz_t_ad_d * az_t_ad_i,
      Qz_mu_d * az_mu_i) +
    c(Qz_t_ad_i * az_t_ad_d,
      Qz_mu_i * az_mu_d) +
    c(Qz_t_ad_i * az_t_ad_i,
      Qz_mu_i * az_mu_i)

  QZAZ_all_ad_d <- sum((QZAZ_all_ad_l + QZAZ_all_ad_u)/2)
  QZAZ_all_ad_i <- sum(abs(QZAZ_all_ad_u - QZAZ_all_ad_l)/2)

  QZAZ_mue_ad_l <- c(Qz_mu_d * az_mu_d)

  QZAZ_mue_ad_u <- QZAZ_mue_ad_l +
    c(Qz_mu_d * az_mu_i) +
    c(Qz_mu_i * az_mu_d) +
    c(Qz_mu_i * az_mu_i)

  QZAZ_mue_ad_d <- sum(QZAZ_mue_ad_l + QZAZ_mue_ad_u)/2
  QZAZ_mue_ad_i <- sum(abs(QZAZ_mue_ad_u - QZAZ_mue_ad_l)/2)

  #### Qz × Ay

  QZAy_all_ad_l <- c(Qz_t_ad_d * ay_t_ad_d,
                     Qz_mu_d * ay_mu_d)

  QZAy_all_ad_u <- QZAy_all_ad_l +
    c(Qz_t_ad_d * ay_t_ad_i,
      Qz_mu_d * ay_mu_i) +
    c(Qz_t_ad_i * ay_t_ad_d,
      Qz_mu_i * ay_mu_d) +
    c(Qz_t_ad_i * ay_t_ad_i,
      Qz_mu_i * ay_mu_i)

  QZAy_all_ad_d <- sum((QZAy_all_ad_l + QZAy_all_ad_u)/2)
  QZAy_all_ad_i <- sum(abs(QZAy_all_ad_u - QZAy_all_ad_l)/2)

  QZAy_mue_ad_l <- c(Qz_mu_d * ay_mu_d)

  QZAy_mue_ad_u <- QZAy_mue_ad_l +
    c(Qz_mu_d * ay_mu_i) +
    c(Qz_mu_i * ay_mu_d) +
    c(Qz_mu_i * ay_mu_i)

  QZAy_mue_ad_d <- sum(QZAy_mue_ad_l + QZAy_mue_ad_u)/2
  QZAy_mue_ad_i <- sum(abs(QZAy_mue_ad_u - QZAy_mue_ad_l)/2)


  #-------------------------------------------------------
  # Estimation of Neutrosophic Regression Coefficients
  #-------------------------------------------------------

  ###### EXX

  EXX_all_ad_l <- zpz_d - QZAZ_all_ad_d
  EXX_all_ad_u <- EXX_all_ad_l + zpz_i - QZAZ_all_ad_i

  EXX_all_ad_d <- (EXX_all_ad_l + EXX_all_ad_u)/2
  EXX_all_ad_i <- abs(EXX_all_ad_u - EXX_all_ad_l)/2

  EXX_mue_ad_l <- zpz_d - QZAZ_mue_ad_d
  EXX_mue_ad_u <- EXX_mue_ad_l + zpz_i - QZAZ_mue_ad_i

  EXX_mue_ad_d <- (EXX_mue_ad_l + EXX_mue_ad_u)/2
  EXX_mue_ad_i <- abs(EXX_mue_ad_u - EXX_mue_ad_l)/2

  ###### EXY

  EXy_all_ad_l <- zpy_d - QZAy_all_ad_d
  EXy_all_ad_u <- EXy_all_ad_l + zpy_i - QZAy_all_ad_i

  EXy_all_ad_d <- (EXy_all_ad_l + EXy_all_ad_u)/2
  EXy_all_ad_i <- abs(EXy_all_ad_u - EXy_all_ad_l)/2

  EXy_mue_ad_l <- zpy_d - QZAy_mue_ad_d
  EXy_mue_ad_u <- EXy_mue_ad_l + zpy_i - QZAy_mue_ad_i

  EXy_mue_ad_d <- (EXy_mue_ad_l + EXy_mue_ad_u)/2
  EXy_mue_ad_i <- abs(EXy_mue_ad_u - EXy_mue_ad_l)/2

  ###### Regression Coefficients

  gamma_1_ad_l <- EXy_all_ad_d / EXX_all_ad_d

  gamma_1_ad_u <- gamma_1_ad_l +
    (EXy_all_ad_d * EXX_all_ad_i +
       EXy_all_ad_i * EXX_all_ad_d +
       EXy_all_ad_i * EXX_all_ad_i) /
    (EXX_all_ad_d * EXX_all_ad_d)

  gamma_1_ad_d <- (gamma_1_ad_l + gamma_1_ad_u)/2
  gamma_1_ad_i <- abs(gamma_1_ad_u - gamma_1_ad_l)/2

  gamma_2_ad_l <- EXy_mue_ad_d / EXX_mue_ad_d

  gamma_2_ad_u <- gamma_2_ad_l +
    (EXy_mue_ad_d * EXX_mue_ad_i +
       EXy_mue_ad_i * EXX_mue_ad_d +
       EXy_mue_ad_i * EXX_mue_ad_i) /
    (EXX_mue_ad_d * EXX_mue_ad_d)

  gamma_2_ad_d <- (gamma_2_ad_l + gamma_2_ad_u)/2
  gamma_2_ad_i <- abs(gamma_2_ad_u - gamma_2_ad_l)/2


  #-------------------------------------------------------
  # Computation of Covariate-Adjusted Effects
  #-------------------------------------------------------

  ###### Gamma × Az

  gaz_all_ad_l <- az_all_ad_d %*% gamma_1_ad_d

  gaz_all_ad_u <- gaz_all_ad_l +
    az_all_ad_d %*% gamma_1_ad_i +
    az_all_ad_i %*% gamma_1_ad_d +
    az_all_ad_i %*% gamma_1_ad_i

  gaz_all_ad_d <- (gaz_all_ad_l + gaz_all_ad_u)/2
  gaz_all_ad_i <- abs(gaz_all_ad_u - gaz_all_ad_l)/2

  acap_all_ad_l <- ay_all_ad_d - gaz_all_ad_d

  acap_all_ad_u <- acap_all_ad_l +
    ay_all_ad_i - gaz_all_ad_i

  acap_all_ad_d <- (acap_all_ad_l + acap_all_ad_u)/2
  acap_all_ad_i <- abs(acap_all_ad_u - acap_all_ad_l)/2


  gaz_mue_ad_l <- az_mue_ad_d %*% gamma_2_ad_d

  gaz_mue_ad_u <- gaz_mue_ad_l +
    az_mue_ad_d %*% gamma_2_ad_i +
    az_mue_ad_i %*% gamma_2_ad_d +
    az_mue_ad_i %*% gamma_2_ad_i

  gaz_mue_ad_d <- (gaz_mue_ad_l + gaz_mue_ad_u)/2
  gaz_mue_ad_i <- abs(gaz_mue_ad_u - gaz_mue_ad_l)/2

  acap_mue_ad_l <- ay_mue_ad_d - gaz_mue_ad_d

  acap_mue_ad_u <- acap_mue_ad_l +
    ay_mue_ad_i - gaz_mue_ad_i

  acap_mue_ad_d <- (acap_mue_ad_l + acap_mue_ad_u)/2
  acap_mue_ad_i <- abs(acap_mue_ad_u - acap_mue_ad_l)/2

  #-------------------------------------------------------
  # Computation of Covariate-Adjusted Sums of Squares
  #-------------------------------------------------------

  acapss_all_ad_l <- Qy_all_ad_d * acap_all_ad_d

  acapss_all_ad_u <- acapss_all_ad_l +
    Qy_all_ad_d * acap_all_ad_i +
    Qy_all_ad_i * acap_all_ad_d +
    Qy_all_ad_i * acap_all_ad_i

  acapss_all_ad_d <- (acapss_all_ad_u + acapss_all_ad_l)/2
  acapss_all_ad_i <- abs(acapss_all_ad_u - acapss_all_ad_l)/2

  acapss_mue_ad_l <- Qy_mue_ad_d * acap_mue_ad_d

  acapss_mue_ad_u <- acapss_mue_ad_l +
    Qy_mue_ad_d * acap_mue_ad_i +
    Qy_mue_ad_i * acap_mue_ad_d +
    Qy_mue_ad_i * acap_mue_ad_i

  acapss_mue_ad_d <- (acapss_mue_ad_u + acapss_mue_ad_l)/2
  acapss_mue_ad_i <- abs(acapss_mue_ad_u - acapss_mue_ad_l)/2

  ss_all_ad_l <- sum(acapss_all_ad_l)
  ss_all_ad_u <- sum(acapss_all_ad_u)
  ss_all_ad_d <- (ss_all_ad_l + ss_all_ad_u)/2
  ss_all_ad_i <- sum(acapss_all_ad_i)

  ss_mue_ad_l <- sum(acapss_mue_ad_l)
  ss_mue_ad_u <- sum(acapss_mue_ad_u)
  ss_mue_ad_d <- (ss_mue_ad_l + ss_mue_ad_u)/2
  ss_mue_ad_i <- sum(acapss_mue_ad_i)

  gzy_1_ad_l <- gamma_1_ad_d * zpy_d

  gzy_1_ad_u <- gzy_1_ad_l +
    gamma_1_ad_d * zpy_i +
    gamma_1_ad_i * zpy_d +
    gamma_1_ad_i * zpy_i

  gzy_1_ad_d <- (gzy_1_ad_l + gzy_1_ad_u)/2
  gzy_1_ad_i <- abs(gzy_1_ad_u - gzy_1_ad_l)/2

  gzy_2_ad_l <- gamma_2_ad_d * zpy_d

  gzy_2_ad_u <- gzy_2_ad_l +
    gamma_2_ad_d * zpy_i +
    gamma_2_ad_i * zpy_d +
    gamma_2_ad_i * zpy_i

  gzy_2_ad_d <- (gzy_2_ad_l + gzy_2_ad_u)/2
  gzy_2_ad_i <- abs(gzy_2_ad_u - gzy_2_ad_l)/2

  mdss_all_ad_l <- ss_all_ad_d + gzy_1_ad_d

  mdss_all_ad_u <- mdss_all_ad_l +
    ss_all_ad_i + gzy_1_ad_i

  mdss_all_ad_d <- (mdss_all_ad_u + mdss_all_ad_l)/2
  mdss_all_ad_i <- abs(mdss_all_ad_u - mdss_all_ad_l)/2

  mdss_mue_ad_l <- ss_mue_ad_d + gzy_2_ad_d

  mdss_mue_ad_u <- mdss_mue_ad_l +
    ss_mue_ad_i + gzy_2_ad_i

  mdss_mue_ad_d <- (mdss_mue_ad_u + mdss_mue_ad_l)/2
  mdss_mue_ad_i <- abs(mdss_mue_ad_u - mdss_mue_ad_l)/2

  #-------------------------------------------------------
  # Computation of Error, Treatment, and Covariate
  # Sums of Squares
  #-------------------------------------------------------

  #### Error Sum of Squares

  ess_all_ad_l <- ypy_d - mdss_all_ad_d
  ess_all_ad_u <- ess_all_ad_l + ypy_i - mdss_all_ad_i

  ess_all_ad_d <- (ess_all_ad_l + ess_all_ad_u)/2
  ess_all_ad_i <- abs(ess_all_ad_u - ess_all_ad_l)/2

  esst_ad_l <- ypy_d - mdss_mue_ad_d
  esst_ad_u <- esst_ad_l + ypy_i - mdss_mue_ad_i

  esst_ad_d <- (esst_ad_u + esst_ad_l)/2
  esst_ad_i <- abs(esst_ad_u - esst_ad_l)/2

  #### Treatment Sum of Squares Adjusted for Covariate

  tss_ad_l <- esst_ad_d - ess_all_ad_d
  tss_ad_u <- tss_ad_l + esst_ad_i - ess_all_ad_i

  tss_ad_d <- (tss_ad_u + tss_ad_l)/2
  tss_ad_i <- abs(tss_ad_u - tss_ad_l)/2

  #### Covariate Sum of Squares

  gss_l <- gamma_1_ad_d * EXy_all_ad_d

  gss_u <- gss_l +
    gamma_1_ad_d * EXy_all_ad_i +
    gamma_1_ad_i * EXy_all_ad_d +
    gamma_1_ad_i * EXy_all_ad_i

  gss_d <- (gss_l + gss_u)/2
  gss_i <- abs(gss_u - gss_l)/2

  #### Total Sum of Squares

  toss_l <- t(dy) %*% dy

  toss_u <- toss_l +
    2 * t(dy) %*% iy +
    t(iy) %*% iy

  toss_d <- (toss_l + toss_u)/2
  toss_i <- abs(toss_u - toss_l)/2

  #-------------------------------------------------------
  # Computation of Interval Mean Squares and
  # F-Statistics
  #-------------------------------------------------------

  SV <- c("Covariate", "Treatment", "Error", "Total")

  DF <- c(1, v - 1, n - v - 1, n - 1)

  SSN <- list(
    c(gss_l, gss_u),
    c(tss_ad_l, tss_ad_u),
    c(ess_all_ad_l, ess_all_ad_u),
    c(toss_l, toss_u)
  )

  MSSN <- list(
    c(gss_l, gss_u),
    c(tss_ad_l/(v - 1), tss_ad_u/(v - 1)),
    c(ess_all_ad_l/(n - v - 1), ess_all_ad_u/(n - v - 1)),
    c(NA, NA)
  )

  F_g_L <- (gss_d/1) /
    (ess_all_ad_d/(n - v - 1))

  F_g_U <- F_g_L +
    ((gss_d * (ess_all_ad_i/(n - v - 1)) +
        gss_i * (ess_all_ad_d/(n - v - 1)) +
        gss_i * (ess_all_ad_i/(n - v - 1))) /
       ((ess_all_ad_d/(n - v - 1)) *
          (ess_all_ad_d/(n - v - 1))))

  F_t_ad_L <- (tss_ad_d/(v - 1)) /
    (ess_all_ad_d/(n - v - 1))

  F_t_ad_U <- F_t_ad_L +
    (((tss_ad_d/(v - 1)) *
        (ess_all_ad_i/(n - v - 1)) +
        (tss_ad_i/(v - 1)) *
        (ess_all_ad_d/(n - v - 1)) +
        (tss_ad_i/(v - 1)) *
        (ess_all_ad_i/(n - v - 1))) /
       ((ess_all_ad_d/(n - v - 1)) *
          (ess_all_ad_d/(n - v - 1))))

  FN <- list(
    c(F_g_L, F_g_U),
    c(F_t_ad_L, F_t_ad_U),
    c(NA, NA),
    c(NA, NA)
  )

  format_interval <- function(x){
    if(any(is.na(x))) return("")
    paste0("[", round(x[1], 2), ", ", round(x[2], 2), "]")
  }

  SSN_fmt <- sapply(SSN, format_interval)
  MSSN_fmt <- sapply(MSSN, format_interval)
  FN_fmt <- sapply(FN, format_interval)

  nancova_table <- data.frame(
    SV = SV,
    DF = DF,
    SSN = SSN_fmt,
    MSSN = MSSN_fmt,
    FN = FN_fmt,
    stringsAsFactors = FALSE
  )

  #-------------------------------------------------------
  # Determination of Significance
  #-------------------------------------------------------

  Ftab_5   <- qf(0.95, 1, n - v - 1)
  Ftab_1   <- qf(0.99, 1, n - v - 1)
  Ftab_001 <- qf(0.999, 1, n - v - 1)

  cov_significance <- ifelse(
    F_g_L > Ftab_001, "***",
    ifelse(
      F_g_L > Ftab_1, "**",
      ifelse(
        F_g_L > Ftab_5, "*", "NS"
      )
    )
  )

  Ftab_5   <- qf(0.95, v - 1, n - v - 1)
  Ftab_1   <- qf(0.99, v - 1, n - v - 1)
  Ftab_001 <- qf(0.999, v - 1, n - v - 1)

  trt_significance <- ifelse(
    F_t_ad_L > Ftab_001, "***",
    ifelse(
      F_t_ad_L > Ftab_1, "**",
      ifelse(
        F_t_ad_L > Ftab_5, "*", "NS"
      )
    )
  )

  nancova_table$Significance <- c(
    cov_significance,
    trt_significance,
    "",
    ""
  )

  #-------------------------------------------------------
  # Interval-Based LSD Analysis
  #-------------------------------------------------------

  if(trt_significance != "NS"){

    dfe <- n - v - 1

    MSE_d <- ess_all_ad_d/dfe
    MSE_i <- ess_all_ad_i/dfe

    SEd_d <- sqrt((2*MSE_d)/r)
    SEd_i <- sqrt((2*MSE_i)/r)

    tval <- qt(1-alpha/2,dfe)

    LSD_l <- tval*SEd_d
    LSD_u <- LSD_l+tval*SEd_i

    adjmeans_d <- acap_all_ad_d[1:v]
    adjmeans_i <- acap_all_ad_i[1:v]

    comparison <- data.frame()

    for(i in 1:(v-1)){

      for(j in (i+1):v){

        Diff_l <- abs(adjmeans_d[i]-adjmeans_d[j])
        Diff_u <- Diff_l+adjmeans_i[i]+adjmeans_i[j]

        Lower <- min(Diff_l,Diff_u)
        Upper <- max(Diff_l,Diff_u)

        if(Upper < LSD_l){

          Result <- "NS"

        }else if(Lower > LSD_u){

          Result <- "S"

        }else{

          Result <- "I"

        }

        comparison <- rbind(
          comparison,
          data.frame(
            Treatment1=i,
            Treatment2=j,
            Difference=paste0("[",
                              round(Lower,2),
                              ", ",
                              round(Upper,2),
                              "]"),
            LSD=paste0("[",
                       round(LSD_l,2),
                       ", ",
                       round(LSD_u,2),
                       "]"),
            Result=Result,
            stringsAsFactors=FALSE
          )
        )

      }

    }

  }else{

    comparison <- NULL
    LSD_l <- NULL
    LSD_u <- NULL

  }

  #-------------------------------------------------------
  # Display Results
  #-------------------------------------------------------

  if(verbose){

    message("\nNeutrosophic Analysis of Covariance Table")
    print(nancova_table,row.names=FALSE)

    if(trt_significance != "NS"){

      message("\nTreatment effect is significant. Hence multiple comparison using LSD is performed.")

      message("\nLSD Interval : [",
              round(LSD_l,4),
              ", ",
              round(LSD_u,4),
              "]")

      message("\nTreatment Comparisons Using LSD")

      print(comparison,row.names=FALSE)

    }else{

      message("\nTreatment effect is non significant. Hence multiple comparison is not performed.")

    }

    message("\nSignificance Codes:")
    message("*** : Significant at p < 0.001")
    message("**  : Significant at p < 0.01")
    message("*   : Significant at p < 0.05")
    message("NS  : Non Significant")
    message("I   : Indeterminate")

  }

  #-------------------------------------------------------
  # Return Results
  #-------------------------------------------------------

  invisible(list(
    nancova_table = nancova_table,
    comparison = comparison,
    LSD = if(!is.null(LSD_l))
      c(Lower=LSD_l,Upper=LSD_u)
    else NULL
  ))

}

#' Aitken Transformation for Neutrosophic Interval Data
#'
#' Applies an Aitken variance-stabilizing transformation to interval-valued
#' observations using group-specific interval MSE values.
#'
#' @param Lower_y Numeric matrix of lower interval bounds.
#' @param Upper_y Numeric matrix of upper interval bounds.
#' @param group Vector identifying the group/year for every row of the input.
#' @param MSE_lower Numeric vector of lower MSE bounds, one per group.
#' @param MSE_upper Numeric vector of upper MSE bounds, one per group.
#'
#' @return A list containing transformed lower/upper matrices and their
#'   determinate and indeterminate components.
#'
#' @examples
#' Lower_y <- matrix(
#'   c(8.2, 5.1,
#'     7.9, 5.4,
#'     9.0, 6.2,
#'     8.8, 6.0,
#'     7.5, 4.8,
#'     7.8, 5.0,
#'     8.6, 5.8,
#'     8.4, 5.6),
#'   ncol = 2,
#'   byrow = TRUE
#' )
#'
#' Upper_y <- matrix(
#'   c(8.8, 5.7,
#'     8.5, 6.0,
#'     9.6, 6.8,
#'     9.4, 6.6,
#'     8.1, 5.4,
#'     8.4, 5.6,
#'     9.2, 6.4,
#'     9.0, 6.2),
#'   ncol = 2,
#'   byrow = TRUE
#' )
#'
#' group <- rep(1:4, each = 2)
#'
#' transformed <- nsAitkenTransform(
#'   Lower_y = Lower_y,
#'   Upper_y = Upper_y,
#'   group = group,
#'   MSE_lower = c(29.83, 16.30, 21.21, 3.62),
#'   MSE_upper = c(30.02, 16.43, 21.24, 3.89)
#' )
#'
#' transformed$weights
#' @export

nsAitkenTransform <- function(Lower_y, Upper_y, group,
                               MSE_lower, MSE_upper) {

  stopifnot(
    is.matrix(Lower_y),
    is.matrix(Upper_y),
    identical(dim(Lower_y), dim(Upper_y)),
    length(group) == nrow(Lower_y)
  )

  if (any(Lower_y > Upper_y)) {
    stop("Each value in Lower_y must be less than or equal to Upper_y.")
  }

  group <- factor(group)
  g <- nlevels(group)

  if (length(MSE_lower) != g || length(MSE_upper) != g) {
    stop("MSE_lower and MSE_upper must each contain one value per group.")
  }

  # Makes the code robust when supplied MSE interval bounds are reversed
  mse_L <- pmin(MSE_lower, MSE_upper)
  mse_U <- pmax(MSE_lower, MSE_upper)

  if (any(mse_L <= 0)) {
    stop("All MSE values must be positive.")
  }

  # Since 1 / sqrt(MSE) is decreasing, transformed weight bounds reverse.
  wt_L <- 1 / sqrt(mse_U)
  wt_U <- 1 / sqrt(mse_L)

  wt_d <- (wt_L + wt_U) / 2
  wt_i <- abs(wt_U - wt_L) / 2

  d_y <- (Lower_y + Upper_y) / 2
  i_y <- abs(Upper_y - Lower_y) / 2

  transformed_data_l <- matrix(
    NA_real_, nrow = nrow(Lower_y), ncol = ncol(Lower_y)
  )
  transformed_data_u <- transformed_data_l

  for (j in seq_len(g)) {
    rows <- which(group == levels(group)[j])

    # Retains your determinate/indeterminate interval arithmetic
    transformed_data_l[rows, ] <- d_y[rows, ] * wt_d[j]

    transformed_data_u[rows, ] <- transformed_data_l[rows, ] +
      d_y[rows, ] * wt_i[j] +
      i_y[rows, ] * wt_d[j] +
      i_y[rows, ] * wt_i[j]
  }

  transform_d <- (transformed_data_l + transformed_data_u) / 2
  transform_i <- abs(transformed_data_u - transformed_data_l) / 2

  list(
    Lower_y = transformed_data_l,
    Upper_y = transformed_data_u,
    determinate = transform_d,
    indeterminate = transform_i,
    weights = data.frame(
      Group = levels(group),
      MSE_lower = mse_L,
      MSE_upper = mse_U,
      Weight_lower = wt_L,
      Weight_upper = wt_U
    )
  )
}

#' Pooled Neutrosophic Analysis of Variance for a Randomized Block Design
#'
#' Performs the pooled neutrosophic ANOVA calculation for interval-valued
#' observations measured in a Randomized Block Design over multiple years or locations or seasons.
#'
#' @param Lower_y Numeric matrix of lower response bounds. Columns must be
#'   ordered in consecutive year blocks, each containing all treatments.
#' @param Upper_y Numeric matrix of upper response bounds, with the same
#'   dimensions as `Lower_y`.
#' @param design Numeric matrix giving the treatment labels for `Lower_y`.
#' @param alpha Significance level for the treatment-by-year interaction test.
#' @param verbose Logical; print the neutrosophic ANOVA table when `TRUE`.
#'
#' @return An invisible list containing the neutrosophic ANOVA table,
#'   interaction F interval and critical value, interaction decision, and the
#'   mean-square interval used for subsequent F intervals.
#'
#' @importFrom MASS ginv
#' @importFrom stats qf
#'
#' @examples
#' # Three blocks, three treatments, and two years.
#' # Columns 1-3 are Year 1; columns 4-6 are Year 2.
#' # Each block contains every treatment once in each year.
#'
#' Lower_y <- matrix(c(
#'   20, 25, 30, 22, 28, 23,
#'   27, 32, 21, 33, 24, 29,
#'   31, 22, 26, 25, 30, 34
#' ), nrow = 3, byrow = TRUE)
#'
#' Upper_y <- matrix(c(
#'   22, 27, 32, 24, 30, 25,
#'   29, 34, 23, 35, 26, 31,
#'   33, 24, 28, 27, 32, 36
#' ), nrow = 3, byrow = TRUE)
#'
#' # Treatment layout for each block:
#' # Year 1 occupies columns 1-3.
#' # Year 2 occupies columns 4-6.
#' design <- matrix(c(
#'   1, 2, 3, 2, 3, 1,
#'   2, 3, 1, 3, 1, 2,
#'   3, 1, 2, 1, 2, 3
#' ), nrow = 3, byrow = TRUE)
#'
#' result <- PooledRBDnsANOVA(
#'   Lower_y = Lower_y,
#'   Upper_y = Upper_y,
#'   design = design,
#'   alpha = 0.05,
#'   verbose = TRUE
#' )
#'
#' result$NANOVA.table
#'
#' @export


PooledRBDnsANOVA <- function(Lower_y, Upper_y, design,
                          alpha = 0.05, verbose = FALSE) {

  Lower_y <- as.matrix(Lower_y)
  Upper_y <- as.matrix(Upper_y)
  design <- as.matrix(design)

  if (!identical(dim(Lower_y), dim(Upper_y)) ||
      !identical(dim(Lower_y), dim(design))) {
    stop("Lower_y, Upper_y, and design must have the same dimensions.")
  }

  if (any(Lower_y > Upper_y)) {
    stop("Each value in Lower_y must be less than or equal to Upper_y.")
  }

  v <- max(design)
  b <- nrow(design)
  k <- ncol(design)

  treatment_counts <- table(design[1, ])
  r <- as.integer(treatment_counts[1])

  if (k != r * v) {
    stop("Columns must be arranged in consecutive year blocks.")
  }

  l <- b * v
  n <- b * k

  trt_vec <- as.vector(t(design))

  Tmat <- matrix(0, n, v)
  for (i in seq_len(n)) {
    Tmat[i, trt_vec[i]] <- 1
  }

  mu <- matrix(1, n, 1)

  blk <- rep(seq_len(l), each = v)
  Bmat <- matrix(0, n, l)
  for (i in seq_len(n)) {
    Bmat[i, blk[i]] <- 1
  }

  rep_vec <- rep(seq_len(r), each = n / r)
  Rmat <- matrix(0, n, r)
  for (i in seq_len(n)) {
    Rmat[i, rep_vec[i]] <- 1
  }

  TYmat <- t(vapply(
    seq_len(n),
    function(i) kronecker(Rmat[i, ], Tmat[i, ]),
    numeric(r * v)
  ))

  d_y <- (Lower_y + Upper_y) / 2
  i_y <- abs(Upper_y - Lower_y) / 2

  dy <- unlist(lapply(
    split(seq_len(ncol(d_y)), ceiling(seq_len(ncol(d_y)) / v)),
    function(j) c(t(d_y[, j, drop = FALSE]))
  ))

  iy <- unlist(lapply(
    split(seq_len(ncol(i_y)), ceiling(seq_len(ncol(i_y)) / v)),
    function(j) c(t(i_y[, j, drop = FALSE]))
  ))

  ybar_d <- mean(d_y)
  ybar_i <- mean(i_y)

  yminusbar_l <- d_y - ybar_d
  yminusbar_u <- yminusbar_l + i_y - ybar_i

  yminusbar_d <- (yminusbar_l + yminusbar_u) / 2
  yminusbar_i <- abs(yminusbar_u - yminusbar_l) / 2

  yminusbar_sq_l <- yminusbar_d^2
  yminusbar_sq_u <- yminusbar_sq_l +
    2 * yminusbar_d * yminusbar_i +
    yminusbar_i^2

  toss_l <- sum(yminusbar_sq_l)
  toss_u <- sum(yminusbar_sq_u)
  toss_d <- (toss_l + toss_u) / 2
  toss_i <- abs(toss_u - toss_l) / 2

  interval_effect <- function(Cmat, C_adjust,
                              Qmat, Q_adjust,
                              rows = NULL) {

    M <- diag(n) -
      C_adjust %*%
      MASS::ginv(t(C_adjust) %*% C_adjust) %*%
      t(C_adjust)

    C <- t(Cmat) %*% M %*% Cmat

    Q_l <- t(Qmat) %*% dy -
      t(Qmat) %*% Q_adjust %*%
      MASS::ginv(t(Q_adjust) %*% Q_adjust) %*%
      t(Q_adjust) %*% dy

    Q_u <- Q_l +
      t(Qmat) %*% iy -
      t(Qmat) %*% Q_adjust %*%
      MASS::ginv(t(Q_adjust) %*% Q_adjust) %*%
      t(Q_adjust) %*% iy

    Q_d <- (Q_l + Q_u) / 2
    Q_i <- abs(Q_l - Q_u) / 2

    if (!is.null(rows)) {
      Q_d <- Q_d[rows, , drop = FALSE]
      Q_i <- Q_i[rows, , drop = FALSE]
    }

    a_l <- MASS::ginv(C) %*% Q_d
    a_u <- a_l + MASS::ginv(C) %*% Q_i

    list(
      Q_d = Q_d,
      Q_i = Q_i,
      a_d = (a_l + a_u) / 2,
      a_i = abs(a_l - a_u) / 2
    )
  }

  trt <- interval_effect(
    Tmat, cbind(mu, Bmat, Rmat),
    cbind(Tmat, Bmat, Rmat), mu,
    seq_len(v)
  )

  blk_effect <- interval_effect(
    Bmat, cbind(mu, Tmat, Rmat),
    cbind(Bmat, Tmat), cbind(mu, Rmat),
    seq_len(l)
  )

  yr <- interval_effect(
    Rmat, cbind(mu, Tmat),
    cbind(Rmat, Bmat, Tmat), mu,
    seq_len(r)
  )

  ty <- interval_effect(
    TYmat, cbind(mu, Tmat, Rmat),
    TYmat, cbind(mu, Tmat, Rmat)
  )

  interval_ss <- function(effect) {

    ss_l <- sum(effect$Q_d * effect$a_d)

    ss_u <- ss_l +
      sum(
        effect$Q_d * effect$a_i +
          effect$Q_i * effect$a_d +
          effect$Q_i * effect$a_i
      )

    list(
      lower = ss_l,
      upper = ss_u,
      d = (ss_l + ss_u) / 2,
      i = abs(ss_u - ss_l) / 2
    )
  }

  ss_t <- interval_ss(trt)
  ss_b <- interval_ss(blk_effect)
  ss_r <- interval_ss(yr)
  ss_ty <- interval_ss(ty)

  mdss_l <- ss_t$d + ss_b$d + ss_r$d + ss_ty$d

  mdss_u <- mdss_l +
    ss_t$i + ss_b$i + ss_r$i + ss_ty$i

  mdss_d <- (mdss_l + mdss_u) / 2
  mdss_i <- abs(mdss_u - mdss_l) / 2

  ess_l <- toss_d - mdss_d
  ess_u <- ess_l + toss_i - mdss_i

  ess_d <- (ess_l + ess_u) / 2
  ess_i <- abs(ess_u - ess_l) / 2

  df_year <- r - 1
  df_trt <- v - 1
  df_rep <- r * (b - 1)
  df_ty <- (v - 1) * (r - 1)
  df_error <- r * (v - 1) * (b - 1)

  F_ty_L <- (ss_ty$d / df_ty) / (ess_d / df_error)

  F_ty_U <- F_ty_L +
    (
      (ss_ty$d / df_trt) * (ess_d / df_error) +
        (ss_ty$i / df_trt) * (ess_d / df_error) +
        (ss_ty$i / df_trt) * (ess_d / df_error)
    ) /
    (
      (ess_d / (ess_d / df_error)) *
        (ess_d / (ess_d / df_error))
    )

  F_critical <- stats::qf(
    1 - alpha,
    df1 = df_ty,
    df2 = df_error
  )

  interaction <- if (F_ty_L > F_critical) {
    "Significant"
  } else if (F_ty_U <= F_critical) {
    "Not Significant"
  } else {
    "Indeterminate"
  }

  if (interaction == "Not Significant") {

    pooled_MSE_l <- (ss_ty$d + ess_d) / (df_ty + df_error)

    pooled_MSE_u <- pooled_MSE_l +
      ss_ty$i / df_ty +
      ess_i / df_error

    pooled_MSE_d <- (pooled_MSE_l + pooled_MSE_u) / 2
    pooled_MSE_i <- abs(pooled_MSE_u - pooled_MSE_l) / 2

  } else {

    pooled_MSE_l <- ss_ty$lower / df_ty
    pooled_MSE_u <- ss_ty$upper / df_ty
    pooled_MSE_d <- ss_ty$d / df_ty
    pooled_MSE_i <- ss_ty$i / df_ty
  }

  F_interval <- function(ss, df) {

    F_L <- (ss$d / df) / pooled_MSE_d

    F_U <- F_L +
      (
        (ss$d / df) * pooled_MSE_i +
          (ss$i / df) * pooled_MSE_d +
          (ss$i / df) * pooled_MSE_i
      ) /
      (pooled_MSE_d * pooled_MSE_d)

    c(F_L, F_U)
  }

  F_year <- F_interval(ss_r, df_year)
  F_trt <- F_interval(ss_t, df_trt)
  F_rep <- F_interval(ss_b, df_rep)

  format_interval <- function(x) {
    if (any(is.na(x))) return("")
    paste0("[", round(x[1], 2), ", ", round(x[2], 2), "]")
  }

  SSN <- list(
    c(ss_r$lower, ss_r$upper),
    c(ss_t$lower, ss_t$upper),
    c(ss_b$lower, ss_b$upper),
    c(ss_ty$lower, ss_ty$upper),
    c(ess_l, ess_u),
    c(NA, NA)
  )

  MSSN <- list(
    c(ss_r$lower / df_year, ss_r$upper / df_year),
    c(ss_t$lower / df_trt, ss_t$upper / df_trt),
    c(ss_b$lower / df_rep, ss_b$upper / df_rep),
    c(ss_ty$lower / df_ty, ss_ty$upper / df_ty),
    c(ess_l / df_error, ess_u / df_error),
    c(NA, NA)
  )

  FN <- list(
    F_year,
    F_trt,
    F_rep,
    c(F_ty_L, F_ty_U),
    c(NA, NA),
    c(NA, NA)
  )

  nanova_table <- data.frame(
    SV = c(
      "Year",
      "Treatment",
      "Replication within year",
      "Treatment*Year",
      "Error",
      "Total"
    ),
    DF = c(
      df_year,
      df_trt,
      df_rep,
      df_ty,
      df_error,
      n - 1
    ),
    SSN = vapply(SSN, format_interval, character(1)),
    MSSN = vapply(MSSN, format_interval, character(1)),
    FN = vapply(FN, format_interval, character(1)),
    stringsAsFactors = FALSE
  )

  if (verbose) {
    print(nanova_table, row.names = FALSE)
    cat("\nInteraction:", interaction, "\n")
    cat("F critical:", round(F_critical, 4), "\n")
  }

  invisible(list(
    nanova_table = nanova_table,
    Interaction = interaction,
    Interaction.F = c(Lower = F_ty_L, Upper = F_ty_U),
    F.critical = F_critical,
    Pooled.MSE = c(Lower = pooled_MSE_l, Upper = pooled_MSE_u)
  ))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PooledRBDnsANOVA.R
\name{PooledRBDnsANOVA}
\alias{PooledRBDnsANOVA}
\title{Pooled Neutrosophic Analysis of Variance for a Randomized Block Design}
\usage{
PooledRBDnsANOVA(Lower_y, Upper_y, design, alpha = 0.05, verbose = FALSE)
}
\arguments{
\item{Lower_y}{Numeric matrix of lower response bounds. Columns must be
ordered in consecutive year blocks, each containing all treatments.}

\item{Upper_y}{Numeric matrix of upper response bounds, with the same
dimensions as \code{Lower_y}.}

\item{design}{Numeric matrix giving the treatment labels for \code{Lower_y}.}

\item{alpha}{Significance level for the treatment-by-year interaction test.}

\item{verbose}{Logical; print the neutrosophic ANOVA table when \code{TRUE}.}
}
\value{
An invisible list containing the neutrosophic ANOVA table,
interaction F interval and critical value, interaction decision, and the
mean-square interval used for subsequent F intervals.
}
\description{
Performs the pooled neutrosophic ANOVA calculation for interval-valued
observations measured in a Randomized Block Design over multiple years or locations or seasons.
}
\examples{
# Three blocks, three treatments, and two years.
# Columns 1-3 are Year 1; columns 4-6 are Year 2.
# Each block contains every treatment once in each year.

Lower_y <- matrix(c(
  20, 25, 30, 22, 28, 23,
  27, 32, 21, 33, 24, 29,
  31, 22, 26, 25, 30, 34
), nrow = 3, byrow = TRUE)

Upper_y <- matrix(c(
  22, 27, 32, 24, 30, 25,
  29, 34, 23, 35, 26, 31,
  33, 24, 28, 27, 32, 36
), nrow = 3, byrow = TRUE)

# Treatment layout for each block:
# Year 1 occupies columns 1-3.
# Year 2 occupies columns 4-6.
design <- matrix(c(
  1, 2, 3, 2, 3, 1,
  2, 3, 1, 3, 1, 2,
  3, 1, 2, 1, 2, 3
), nrow = 3, byrow = TRUE)

result <- PooledRBDnsANOVA(
  Lower_y = Lower_y,
  Upper_y = Upper_y,
  design = design,
  alpha = 0.05,
  verbose = TRUE
)

result$NANOVA.table

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CRDnsANOVA.R
\name{CRDnsANOVA}
\alias{CRDnsANOVA}
\title{Neutrosophic Analysis of Variance for Completely Randomized Design}
\usage{
CRDnsANOVA(Lower_y, Upper_y, design, alpha = 0.05, verbose = FALSE)
}
\arguments{
\item{Lower_y}{Matrix of lower bounds of the response variable.}

\item{Upper_y}{Matrix of upper bounds of the response variable.}

\item{design}{Matrix specifying treatment allocation.}

\item{alpha}{Significance level for LSD test.}

\item{verbose}{Logical. If TRUE, prints the analysis.}
}
\value{
A list containing the NANOVA table, treatment means,
pairwise comparisons, and LSD interval.
}
\description{
Performs neutrosophic analysis of variance (NANOVA) for a
completely randomized design using interval-valued observations.
}
\examples{
Lower_y <- matrix(c(
  9.5, 19.5, 29.5, 39.5,
  10.0, 20.0, 30.0, 40.0,
  10.5, 20.5, 30.5, 40.5,
  9.8, 19.8, 29.8, 39.8,
  10.2, 20.2, 30.2, 40.2
), nrow = 5, byrow = TRUE)

Upper_y <- matrix(c(
  10.5, 20.5, 30.5, 40.5,
  11.0, 21.0, 31.0, 41.0,
  11.5, 21.5, 31.5, 41.5,
  10.8, 20.8, 30.8, 40.8,
  11.2, 21.2, 31.2, 41.2
), nrow = 5, byrow = TRUE)

design <- matrix(c(
  1, 2, 3, 4,
  1, 2, 3, 4,
  1, 2, 3, 4,
  1, 2, 3, 4,
  1, 2, 3, 4
), nrow = 5, byrow = TRUE)

CRDnsANOVA(
  Lower_y = Lower_y,
  Upper_y = Upper_y,
  design = design,
  alpha = 0.05,
  verbose = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PooledCRDnsANOVA.R
\name{PooledCRDnsANOVA}
\alias{PooledCRDnsANOVA}
\title{Pooled Neutrosophic ANOVA for a Completely Randomized Design}
\usage{
PooledCRDnsANOVA(Lower_y, Upper_y, design, alpha = 0.05, verbose = FALSE)
}
\arguments{
\item{Lower_y}{Numeric matrix of lower response bounds. Columns must be
arranged in consecutive year or location blocks, each containing all treatments.}

\item{Upper_y}{Numeric matrix of upper response bounds.}

\item{design}{Numeric treatment-layout matrix with the same dimensions as
\code{Lower_y}.}

\item{alpha}{Significance level. Default is \code{0.05}.}

\item{verbose}{Logical; print results when \code{TRUE}.}
}
\value{
A list containing the initial and final NANOVA tables, interaction
test, decision, and pooled MSE when applicable.
}
\description{
Performs pooled neutrosophic analysis of variance for a completely
randomized design.
}
\examples{
# Three replications, three treatments, and two years.
# Columns 1-3 are Year 1; columns 4-6 are Year 2.

Lower_y <- matrix(c(
  20, 25, 30, 21, 26, 31,
  22, 27, 32, 23, 28, 33,
  21, 26, 31, 22, 27, 32
), nrow = 3, byrow = TRUE)

Upper_y <- matrix(c(
  22, 27, 32, 23, 28, 33,
  24, 29, 34, 25, 30, 35,
  23, 28, 33, 24, 29, 34
), nrow = 3, byrow = TRUE)

# Treatment layout:
# Year 1 = Treatments 1, 2, 3
# Year 2 = Treatments 1, 2, 3
design <- matrix(c(
  1, 2, 3, 1, 2, 3,
  1, 2, 3, 1, 2, 3,
  1, 2, 3, 1, 2, 3
), nrow = 3, byrow = TRUE)

result <- PooledCRDnsANOVA(
  Lower_y = Lower_y,
  Upper_y = Upper_y,
  design = design,
  alpha = 0.05,
  verbose = TRUE
)

result$NANOVA.table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nsAitkenTransform.R
\name{nsAitkenTransform}
\alias{nsAitkenTransform}
\title{Aitken Transformation for Neutrosophic Interval Data}
\usage{
nsAitkenTransform(Lower_y, Upper_y, group, MSE_lower, MSE_upper)
}
\arguments{
\item{Lower_y}{Numeric matrix of lower interval bounds.}

\item{Upper_y}{Numeric matrix of upper interval bounds.}

\item{group}{Vector identifying the group/year for every row of the input.}

\item{MSE_lower}{Numeric vector of lower MSE bounds, one per group.}

\item{MSE_upper}{Numeric vector of upper MSE bounds, one per group.}
}
\value{
A list containing transformed lower/upper matrices and their
determinate and indeterminate components.
}
\description{
Applies an Aitken variance-stabilizing transformation to interval-valued
observations using group-specific interval MSE values.
}
\examples{
Lower_y <- matrix(
  c(8.2, 5.1,
    7.9, 5.4,
    9.0, 6.2,
    8.8, 6.0,
    7.5, 4.8,
    7.8, 5.0,
    8.6, 5.8,
    8.4, 5.6),
  ncol = 2,
  byrow = TRUE
)

Upper_y <- matrix(
  c(8.8, 5.7,
    8.5, 6.0,
    9.6, 6.8,
    9.4, 6.6,
    8.1, 5.4,
    8.4, 5.6,
    9.2, 6.4,
    9.0, 6.2),
  ncol = 2,
  byrow = TRUE
)

group <- rep(1:4, each = 2)

transformed <- nsAitkenTransform(
  Lower_y = Lower_y,
  Upper_y = Upper_y,
  group = group,
  MSE_lower = c(29.83, 16.30, 21.21, 3.62),
  MSE_upper = c(30.02, 16.43, 21.24, 3.89)
)

transformed$weights
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nsLevenesTest.R
\name{nsLeveneTest}
\alias{nsLeveneTest}
\title{Neutrosophic Levene's Test for Interval-Valued Data}
\usage{
nsLeveneTest(Lower_y, Upper_y, design, alpha = 0.05, verbose = TRUE)
}
\arguments{
\item{Lower_y}{Numeric matrix of lower interval bounds.}

\item{Upper_y}{Numeric matrix of upper interval bounds; same dimensions as
\code{Lower_y}.}

\item{design}{Treatment-design matrix. Treatment labels in each row must be
arranged in consecutive year blocks.}

\item{alpha}{Significance level. Default is \code{0.05}.}

\item{verbose}{Logical; print the test table when \code{TRUE}.}
}
\value{
An invisible list containing the Levene statistic interval,
critical F value, homogeneity decision, and ANOVA table.
}
\description{
Tests homogeneity of variances for interval-valued observations using
neutrosophic lower and upper test-statistic bounds.
}
\examples{
# Two replications, three treatments, and two years.
# Columns are ordered as:
# Year 1 (T1, T2, T3), Year 2 (T1, T2, T3).

Lower_y <- matrix(c(
  10, 12, 11, 13, 14, 12,
   9, 11, 10, 12, 13, 11
), nrow = 2, byrow = TRUE)

Upper_y <- matrix(c(
  12, 14, 13, 15, 16, 14,
  11, 13, 12, 14, 15, 13
), nrow = 2, byrow = TRUE)

design <- matrix(c(
  1, 2, 3, 1, 2, 3,
  1, 2, 3, 1, 2, 3
), nrow = 2, byrow = TRUE)

levene_result <- nsLeveneTest(
  Lower_y = Lower_y,
  Upper_y = Upper_y,
  design = design,
  alpha = 0.05,
  verbose = TRUE
)

levene_result$Homogeneity

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RBDnsANOVA.R
\name{RCBDnsANOVA}
\alias{RCBDnsANOVA}
\title{Neutrosophic Analysis of Variance for Randomized Complete Block Design}
\usage{
RCBDnsANOVA(Lower_y, Upper_y, design, alpha = 0.05, verbose = FALSE)
}
\arguments{
\item{Lower_y}{Numeric matrix of lower bounds of the response variable.}

\item{Upper_y}{Numeric matrix of upper bounds of the response variable.}

\item{design}{Numeric matrix representing the RCBD layout.}

\item{alpha}{Significance level for interval-based LSD test. Default is 0.05.}

\item{verbose}{Logical. If \code{TRUE}, prints the NANOVA table and LSD comparisons.}
}
\value{
A list containing the Neutrosophic ANOVA table, interval-based LSD
comparisons (if applicable), and the interval LSD.
}
\description{
Performs Neutrosophic Analysis of Variance (NANOVA) for interval-valued
response data from a Randomized Complete Block Design (RCBD).
}
\examples{
Lower_y <- matrix(c(
  120.230,125.488,132.987,127.086,127.672,128.013,
  122.594,121.009,123.969,120.358,120.424,122.197,
  121.183,130.671,128.794,114.863,122.595,122.073,
  127.620,124.532,132.893,125.528,125.850,127.550
), nrow = 4, byrow = TRUE)

Upper_y <- matrix(c(
  127.6967536,131.2116955,141.2127373,136.1540904,130.6884772,136.8474149,
  129.8264289,130.3314544,133.3113414,126.5063118,128.4362999,130.2714433,
  124.5068016,139.3287297,134.1060197,124.2774447,127.2248520,130.3948469,
  131.0638721,129.8884785,135.5666716,127.7580663,132.0178679,133.3903886
), nrow = 4, byrow = TRUE)

design <- matrix(c(
  1,2,3,4,5,6,
  1,2,3,4,5,6,
  1,2,3,4,5,6,
  1,2,3,4,5,6
), nrow = 4, byrow = TRUE)

RCBDnsANOVA(Lower_y, Upper_y, design)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CRDnsANCOVA.R
\name{CRDnsANCOVA}
\alias{CRDnsANCOVA}
\title{Neutrosophic Analysis of Covariance for Completely Randomized Design}
\usage{
CRDnsANCOVA(
  Lower_y,
  Upper_y,
  Lower_z,
  Upper_z,
  design,
  alpha = 0.05,
  verbose = FALSE
)
}
\arguments{
\item{Lower_y}{Numeric matrix of lower bounds of the response variable.}

\item{Upper_y}{Numeric matrix of upper bounds of the response variable.}

\item{Lower_z}{Numeric matrix of lower bounds of the covariate.}

\item{Upper_z}{Numeric matrix of upper bounds of the covariate.}

\item{design}{Numeric matrix representing the CRD treatment layout.}

\item{alpha}{Significance level for interval-based LSD comparisons. Default is 0.05.}

\item{verbose}{Logical. If \code{TRUE}, prints the NANCOVA table and LSD comparisons.}
}
\value{
A list containing the Neutrosophic ANCOVA table, interval LSD comparisons
(if treatment effects are significant), and the interval LSD value.
}
\description{
Performs Neutrosophic Analysis of Covariance (NANCOVA) for interval-valued
response and covariate data from a Completely Randomized Design (CRD).
}
\examples{
Lower_y <- matrix(c(
  20,25,30,35,
  21,26,31,36,
  19,24,29,34,
  20,25,30,35,
  21,26,31,36
), nrow = 5, byrow = TRUE)

Upper_y <- matrix(c(
  22,27,32,37,
  23,28,33,38,
  21,26,31,36,
  22,27,32,37,
  23,28,33,38
), nrow = 5, byrow = TRUE)

Lower_z <- matrix(c(
  8,9,10,11,
  9,10,11,12,
  8,9,10,11,
  9,10,11,12,
  8,9,10,11
), nrow = 5, byrow = TRUE)

Upper_z <- matrix(c(
  9,10,11,12,
  10,11,12,13,
  9,10,11,12,
  10,11,12,13,
  9,10,11,12
), nrow = 5, byrow = TRUE)

design <- matrix(c(
  1,2,3,4,
  1,2,3,4,
  1,2,3,4,
  1,2,3,4,
  1,2,3,4
), nrow = 5, byrow = TRUE)

CRDnsANCOVA(Lower_y, Upper_y, Lower_z, Upper_z, design)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RBDnsANCOVA.R
\name{RCBDnsANCOVA}
\alias{RCBDnsANCOVA}
\title{Neutrosophic Analysis of Covariance for RCBD}
\usage{
RCBDnsANCOVA(
  Lower_y,
  Upper_y,
  Lower_z,
  Upper_z,
  design,
  alpha = 0.05,
  verbose = FALSE
)
}
\arguments{
\item{Lower_y}{Numeric matrix of lower response values.}

\item{Upper_y}{Numeric matrix of upper response values.}

\item{Lower_z}{Numeric matrix of lower covariate values.}

\item{Upper_z}{Numeric matrix of upper covariate values.}

\item{design}{Numeric matrix representing the RCBD layout.}

\item{alpha}{Significance level for LSD test. Default is \code{0.05}.}

\item{verbose}{Logical; if \code{TRUE}, prints the analysis. Default is \code{FALSE}.}
}
\value{
A list containing the NANCOVA table, LSD interval, and treatment
comparisons (if treatment effect is significant).
}
\description{
Performs neutrosophic analysis of covariance for a Randomized Complete
Block Design using interval-valued response and covariate data.
}
\examples{
Lower_y <- matrix(c(
  46.84, 97.69, 39.30, 49.20,
  51.52, 83.38, 56.48, 42.15,
  44.42, 49.64, 44.21, 35.79,
  32.77, 74.30, 55.72, 70.55
), nrow = 4, byrow = TRUE)

Upper_y <- matrix(c(
  49.66,101.31,47.70,55.80,
  60.98,87.62,60.52,44.85,
  52.26,59.36,52.79,44.61,
  42.23,82.70,59.28,75.45
), nrow = 4, byrow = TRUE)

Lower_z <- matrix(c(
  224.95,245.87,245.19,259.41,
  222.06,213.83,253.10,247.51,
  255.67,230.55,265.05,246.67,
  137.98,214.68,251.49,257.70
), nrow = 4, byrow = TRUE)

Upper_z <- matrix(c(
  229.05,250.13,252.81,268.59,
  229.94,222.17,258.90,256.49,
  262.33,237.45,274.95,249.33,
  142.02,219.32,260.51,264.30
), nrow = 4, byrow = TRUE)

design <- matrix(c(
  1,2,3,4,
  1,2,3,4,
  1,2,3,4,
  1,2,3,4
), nrow = 4, byrow = TRUE)

result <- RCBDnsANCOVA(
  Lower_y,
  Upper_y,
  Lower_z,
  Upper_z,
  design,
  alpha = 0.05,
  verbose = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CRDnsMANOVA.R
\name{CRDnsMANOVA}
\alias{CRDnsMANOVA}
\title{Neutrosophic MANOVA for a Completely Randomized Design}
\usage{
CRDnsMANOVA(Lower_y, Upper_y, group, alpha = 0.05, verbose = TRUE)
}
\arguments{
\item{Lower_y}{A numeric matrix containing the lower bounds of the response
variables. Rows represent observations and columns represent response
variables.}

\item{Upper_y}{A numeric matrix containing the upper bounds of the response
variables. Must have the same dimensions as \code{Lower_y}.}

\item{group}{A vector or factor identifying the treatment group for each
observation. Its length must equal \code{nrow(Lower_y)}.}

\item{alpha}{Significance level used to obtain the critical F value.
Defaults to \code{0.05}.}

\item{verbose}{Logical; if \code{TRUE}, prints the neutrosophic MANOVA table.
Defaults to \code{TRUE}.}
}
\value{
A list containing the results of the neutrosophic MANOVA with the following
components:
\describe{
\item{MANOVA.table}{A data frame containing the neutrosophic MANOVA table,
including the source of variation, degrees of freedom, Wilks' Lambda
interval, approximate F-statistic interval, and decision.}
\item{Wilks.Lambda}{A named numeric vector containing the lower and upper
bounds of Wilks' Lambda.}
\item{Approx.F}{A named numeric vector containing the lower and upper
bounds of the approximate F-statistic.}
\item{F.critical}{The critical F value at the specified significance
level.}
\item{Decision}{The hypothesis testing decision ("Significant",
"Not Significant", or "Indeterminate").}
\item{treatment_ss}{A list containing the lower, upper, determinate, and
indeterminate treatment sum of squares matrices.}
\item{total_ss}{A list containing the lower, upper, determinate, and
indeterminate total sum of squares matrices.}
\item{residual_ss}{A list containing the lower, upper, determinate, and
indeterminate residual sum of squares matrices.}
}
}
\description{
Performs neutrosophic multivariate analysis of variance (MANOVA) for
interval-valued multivariate observations under a completely randomized
design using Wilks' Lambda as the multivariate test criterion.
}
\examples{
YL <- matrix(c(8.27, 2.50, 5.30, 1.41,
               8.40, 6.27, 0.00, 3.42),
             ncol = 2, byrow = TRUE)

YU <- matrix(c(9.73, 3.50, 6.70, 2.59,
               9.60, 7.73, 0.00, 4.58),
             ncol = 2, byrow = TRUE)

g <- c(1, 1, 2, 2)

fit <- CRDnsMANOVA(
  Lower_y = YL,
  Upper_y = YU,
  group = g
)

fit$MANOVA.table
fit$Decision
}
